module.exports = function(grunt){

    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        // Running prepare after all changes means that ripple can be reloaded
        // just like a normal page
        exec:{
            prepare: {
                command: "cordova prepare"
            }
        },

        // CSS without a preprocessor is painful
        less: {
            dev: {
                files: {
                    'www/css/index.css': 'www/css/index.less',
                    'www/css/mobile.css': 'www/css/mobile.less',
                    'www/css/question.css': 'www/css/question.less'
                }
            }
        },

        // Notifications make life easier
        notify: {
            watch: {
                options: {
                    title: 'RDY',  // optional
                    message: 'You can reload the browser now', //required
                }
            },
            mash: {
                options: {
                    title: 'Mashed!',
                    message: 'Super-mashed, really'
                }
            }
        },

        // Organize javascripts for speed and ease
        uglify: {
            dev: {
                options: {
                    beautify: true,
                    mangle: false,
                    compress: false
                },
                files: {
                    'www/js/script.js': [
                        // Files will be compressed in the order they are declared.
                        'www/js/libraries/jquery.js',
                        'www/js/libraries/knockout.js',
                        'www/js/libraries/iscroll-probe.js',
                        'www/js/libraries/finch.js',
                        'www/js/libraries/hmac-sha256.js',

                        'www/js/source/boot.js',
                        'www/js/source/storage.js',
                        'www/js/source/picker.js',
                        'www/js/source/site.js',
                        'www/js/source/helpers.js',
                        'www/js/source/registration.js',
                        'www/js/source/beacons.js',
                        'www/js/source/ahoy.js',

                        'settings.js',
                        'www/js/source/app.js',
                        'www/js/viewmodels/*.js',
                    ]
                }
            },
            mash: {
                options: {
                    beautify: false,
                    mangle: true,
                    compress: true
                },
                files: {
                    'www/js/script.js': [
                        // Files will be compressed in the order they are declared.
                        'www/js/libraries/jquery.js',
                        'www/js/libraries/knockout.js',
                        'www/js/libraries/iscroll-probe.js',
                        'www/js/libraries/finch.js',
                        'www/js/libraries/hmac-sha256.js',

                        'www/js/source/boot.js',
                        'www/js/source/storage.js',
                        'www/js/source/picker.js',
                        'www/js/source/site.js',
                        'www/js/source/helpers.js',
                        'www/js/source/registration.js',
                        'www/js/source/beacons.js',
                        'www/js/source/ahoy.js',

                        'settings.js',
                        'www/js/source/app.js',
                        'www/js/viewmodels/*.js',
                    ]
                }
            }
        },

        // Template concatination and manipulation
        htmlbuild: {
            dist: {
                src: 'www/templates/index.html',
                dest: 'www/',
                options: {
                    beautify: false,
                    sections: {
                        views: 'www/templates/views/*.html'
                    }
                }
            }
        },

        // All the things we want to watch
        watch:{
            less: {
                files: ['www/css/*.less'],
                tasks: ['less', 'exec', 'notify:watch']
            },
            scripts: {
                files: ['www/js/libraries/*.js', 'www/js/source/*.js', 'www/js/viewmodels/*.js'],
                tasks: ['uglify:dev', 'exec', 'notify:watch']
            },
            templates: {
                files: [
                    'www/templates/*.html',
                    'www/templates/views/*.html'
                ],
                tasks: ['htmlbuild:dist', 'exec', 'notify:watch']
            },
            // inception
            grunt: {
                files: ['Gruntfile.js']
            }
        }
    });

    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-html-build');
    grunt.loadNpmTasks('grunt-exec');
    grunt.loadNpmTasks('grunt-notify');

    grunt.registerTask('default', ['less', 'uglify:dev', 'htmlbuild:dist', 'notify:watch', 'exec', 'watch']);
    grunt.registerTask('compile', ['less', 'uglify:dev', 'htmlbuild:dist', 'notify:watch', 'exec']);
    grunt.registerTask('mash', ['less', 'uglify:mash', 'htmlbuild:dist', 'exec', 'notify:mash']);
};