// Application Constructor
app.initialize = function() {
    // @todo: this needs to be moved into this.onDeviceReady, it shouldn't fire until Cordova is gtg
    if(app.standalone) {
        storage.set('site_id', app.standalone);
        if(storage.get('site_blob') == null) storage.set('site_blob', app.standalone);
    }

    Date.prototype.addHours= function(h){
        this.setHours(this.getHours()+h);
        return this;
    }

    this.generalBindings();
    this.bindEvents();
};

// Bind Event Listeners
app.bindEvents = function() {
    document.addEventListener('deviceready', this.onDeviceReady, false);
    document.addEventListener('offline', helper.noData, false);
    document.addEventListener('online', helper.hasData, false);
    document.addEventListener('resume', helper.resume, false);
    document.addEventListener('pause', helper.pause, false)
    app.setAjaxPrefilter();
};

app.smallLoad = function(state){
    // state ? $('.spinner').addClass('rightIn') : $('.spinner').removeClass('rightIn');
};

/**
 * Misc DOM bindings
 * @return {[type]}
 */
app.generalBindings = function() {
    // Rebind iScroll, helps keep things consistant.
    if(typeof window.myScroll !== "undefined" && window.myScroll !== null) {
        window.myScroll.destroy();
        window.myScroll = null;
        $("#scroller").attr("style", "");
    }
    window.myScroll = new IScroll(document.getElementById("main"), {
           scrollbars  : true
        ,  mouseWheel  : true
        ,  interactiveScrollbars: true
        ,  click: true
        ,  checkDOMChanges: false
        ,  bounce: false
        ,  probeType: 3
        ,  onBeforeScrollStart: function (e) {
            var target = e.target;
            while (target.nodeType != 1) target = target.parentNode;

            if (target.tagName != 'SELECT' && target.tagName != 'INPUT' && target.tagName != 'TEXTAREA') {
                e.preventDefault();
            }
           }
    });

    helper.platform() === "ios" ? $('#fixedNav').css({"top": "20px"}) : "";

    window.myScroll.on('scroll', helper.scrollCalc);

    window.myScroll.on('scrollEnd', function(){
        delay = helper.reloadView ? 400 : 0;
        setTimeout(function(){
            if(app.State.hasOwnProperty('body') && app.State.body.pullRefresh && app.State.body.refreshOffset() !== 0){
                app.State.body.refreshOffset(0);
                helper.refreshFontSize(2);
            }

            if(helper.reloadView) {
                helper.refreshView();
            }
        }, delay);
    });

    // Generic ajax form handler.
    $("#main").on("submit", ".ajax-form", function(e){

        var $this = $(this),
            data  = $this.serializeObject(),
            endpoint = $this.attr("action"),
            message  = $this.find(".message");

        e.preventDefault();

        app.State.overLoad(true);

        // Prevent multiple submits
        if(window.form_being_submitted) {
            return;
        }
        window.form_being_submitted = true;

        if(message) { message.hide(); }

        ahoy.track("$submit", {
           endpoint: endpoint,
           tag: $this.prop("tagName"),
           id: $this.prop("id")
        });

        $.ajax(
            endpoint,
            {
                'type': 'POST',
                'data': data,
                'crossDomain': true,
                'dataType': 'json',
                'success': function(d) {
                    window.form_being_submitted = false;

                    helper.log("FORM RESPONSE: ",  d);

                    if(d.message) {
                        helper.alertModal(d.message);
                    }

                },
                'error': function(det) {
                    window.form_being_submitted = false;
                    helper.log(det);
                    helper.alertModal('Something went wrong, please try again later.', 'fa-exclamation-triangle');
                },
                'complete': function() {
                    app.State.overLoad(false);
                }
            }
        );
    })
    .on('change', '.fake-checkbox input', function(event){
        var $self = $(this),
            $mockup = $self.parents('.fake-checkbox');

        if($self.is(':checked')) {
            $mockup.addClass('checked');
        }
        else {
            $mockup.removeClass('checked');
        }
    });

    // Prevent the body from being scrolled around, makes the app feel 'solid'
    $(document.body).bind('touchmove', function(event) {
        event.preventDefault();
    });
}

// Toggle the loading class
app.loading = function(toggle) {
    if(toggle) {
        helper.isLoading(true);
        $('body').addClass('loading');
    }
    else {
        helper.isLoading(false);
        $('body').removeClass('loading');
    }
}

// A history of our navigation in the app, with empty items to start
app.history = [
    {'name': null, 'data': {}},
    {'name': null, 'data': {}}
];

// This is only used to check if its function is busy and its only needed for iOS
app.permissionsAsk = cordova.plaformId === "ios" ? true : false;

// Switch to a new view model
app.Display = function(name, data) {
    if(helper.site_disabled(helper.site_blob())){ name = 'disabled'; }

    $('#statusBarColor').fadeOut();
    helper.log('Displaying: ' + name);

    $('#nav').css("visibility","visible");
    $('#fixedNav').css("z-index","0");

    $('#nav').css({"visibility":"visible"});
    $('#fixedNav-container').css({"z-index":"0", "visibility": "hidden"});

    app.State.alertbar.display(false);
    app.State.overLoad(false);
    helper.refreshFontSize(2);

    switch(name) {
        case 'upcoming':
        case 'prizes':
        case 'settings':
        case 'points':
        case 'activity':
        case 'promos':

            helper.pullIncrement = 1;
            helper.originalRefreshOffset = 0;

            // Refresh our user's address info
            app.getAddress();

            // Checks for missing data in our storage variables that are used in functions around the app.
            helper.userMissingData();

            // Check if we have asked for permissions and does so if not.
            helper.checkPermissions();

            // Check if we have an active question and display it if we do.
            helper.log(helper.permissionsRequestRequired(), 'permissions required?');
            if(!helper.permissionsRequestRequired()) {
                helper.checkQuestion();
            }
        break;
        default:
            app.loading(true);
        break;
    }


    // Loading is always set to true, re-bind if things need to be, and free
    // up the main window binding
    app.generalBindings();
    ko.cleanNode(document.getElementById("scroller"));

    // Initialize the body View Model and bind it to the scroller
    app.State.body = new app.ViewModels[name](data, app.State);
    ko.applyBindings(app.State.body, document.getElementById("scroller"));

    // Apply any designated layout changes
    app.layout(name);

    // Tell the nav which page we're displaying
    app.State.nav.activate_page(name);
    // app.State.footer.activate_page(name);

    // This thing always needs to be refreshed after any kind of navigation or page size change
    app.generalBindings();

    // Tell analytics that we've shown a page
    site = "none selected";
    if(site_blob = storage.get('site_blob') !== null) {
        site = site_blob.slug;
        ahoy.doVisitStuff();
    }

    ahoy.track("$view", {
        page: name
    });

    // Lastly, we add this navigation event to our history. I'm not sure if this could happen,
    // but we'll also *not* push an item into history if it's already the last page in history.
    // No endless loops here, please.
    if(helper.this_page().name !== name) {
        app.history.push({ 'name': name, 'data': data });
    }
    if(app.history.length > 10) {
        app.history.splice(0, 1);
    }

    navBottom = $('#nav').offset().top;
    $("#overSpinner").css({ "top":navBottom, "bottom":"0" });
}

/**
 * Manipulate the page layout according to types. I'd rather have this mess in one spot
 * than in each individual controller, even if this method does get big.
 *
 * @param  string type = viewmodel name
 */
app.layout = function(type) {
    var $main = $("#main"),
        $body = $("body");
    // Basic resets
    $main.removeClass("header footer");

    // Note: window.device does not become available until some time after ondeviceready is fired. Luckily
    // it is available by the time app.layout is initialized, but if you have some kind of weird situation
    // where window.device is undefined, you might want to try wrapping it in a setTimeout.
    if(window.device.platform === "iOS" && parseInt(window.device.version) >= 7) {
        $body.addClass("ios7");
    }


    switch(type) {
        case "authenticate":
            app.State.header.visible(false);
            app.State.header.button.visible(false);
            app.State.ad.visible(false);

            // app.State.footer.visible(false);
            // app.State.footer.navigation(false);

            // $main.addClass("header").removeClass("footer header");
            break;

        case "login":
            app.State.header.button.text("Cancel");
            app.State.header.button.visible(true);
            app.State.header.visible(false);
            app.State.ad.visible(false);

            // app.State.footer.visible(false);
            // app.State.footer.navigation(false);

            $main
                // .addClass("header")
                .css({'overflow': 'visible'});
            helper.refreshScroll();
            break;

        case "change_password":
        case "register":
            app.State.header.button.text("Cancel");
            app.State.header.button.visible(true);
            app.State.header.visible(false);
            app.State.ad.visible(false);

            // app.State.footer.visible(false);
            // app.State.footer.navigation(false);

            $main
                // .addClass("header")
                .css({'overflow': 'visible'});

            helper.refreshScroll();
            break;
        case "welcome":
            app.State.header.button.visible(false);
            app.State.header.visible(true);
            app.State.ad.visible(false);

            // app.State.footer.visible(false);
            // app.State.footer.navigation(false);

            // $main.addClass("header footer");
            if(helper.adAvailable()) { $main.addClass("footer"); }
            helper.refreshScroll();
            break;
        case "error_panel":
        case "picker":
            app.State.header.visible(false);
            // app.State.footer.visible(false);
            app.State.ad.visible(false);
            break;

        case "settings":
            app.State.header.button.visible(false);
            app.State.header.visible(true);
            app.State.ad.visible(false);

            // app.State.footer.navigation(true);
            // app.State.footer.visible(true);

            // $main.addClass("header footer");
            if(helper.adAvailable()) { /* $main.addClass("footer"); */ }
            helper.refreshScroll();
            break;
        case "disabled":
            app.State.header.button.visible(false);
            app.State.header.visible(false);
            app.State.ad.visible(false);
            helper.refreshScroll();
            break;
         case "prize":
            app.State.header.button.visible(false);
            app.State.header.visible(false);
            app.State.ad.visible(false);

            if(helper.platform() === "ios") {
                $(".full-topbar").css({"padding":"31px 0 5px"});
            }

            setTimeout(function() {
                helper.refreshScroll();
            }, 100);

            helper.log("They see me scrollin, they reloadin");

            break;
        default:
            app.State.header.button.visible(false);
            app.State.header.visible(true);
            app.State.ad.visible(true);

            // app.State.footer.navigation(true);
            // app.State.footer.visible(true);

            // $main.addClass("header footer");
            if(helper.adAvailable()) { $main.addClass("footer"); }
            helper.refreshScroll();
            break;
    }
};

app.defaultAjaxParams = function() {
    var defaultData = {};

    // Always send a client var so the server can give us phonegap-compatable responses
    defaultData.client = 'phonegap';

    if(storage.get('site_id') !== null) {
        defaultData.site_id = storage.get('site_id');
    }

    if(storage.get('r_users') !== null) {
        defaultData.r_users = storage.get('r_users');
    }

    if(helper.user_id() !== null) {
        defaultData.user_id = helper.user_id();
    }

    if(storage.get('ahoy_visit') !== null) {
        defaultData.visit_token = storage.get('ahoy_visit');
    }

    defaultData.app_version = app.version;

    return defaultData;
}

/**
 * Set some default data to always send with all AJAX requests, if those IDs are set locally
 */
app.setAjaxPrefilter = function() {
    $.ajaxPrefilter(function (options, originalOptions, jqXHR) {

        helper.log([options, originalOptions, jqXHR], 'I NEED TO KNOW THESE THINGS');

        defaultData = app.defaultAjaxParams();

        if(options.contentType === "application/json") {
            options.data = JSON.stringify($.extend(originalOptions.data, defaultData));

            // try { options.data = JSON.stringify($.extend(JSON.parse(originalOptions.data), defaultData)); }
            // catch(e) {options.data = JSON.stringify($.extend(originalOptions.data, defaultData));}
        }
        else {
            options.data = $.param($.extend(originalOptions.data, defaultData));
        }
    });
};


app.getAddress = function() {
    var last_time = storage.get('last_address_refresh_time'),
        now = new Date().getTime(),
        one_week = 86400000*7;

    if(last_time == null || isNaN(last_time)) {
        last_time = now - (one_week * 2);
        storage.set('last_address_refresh_time', last_time);
    }

    helper.log(["TIME", parseInt(last_time) + one_week < now, parseInt(last_time), isNaN(last_time), one_week, now]);


    if(parseInt(last_time) + one_week < now) {
        // Check if the user has an address on file.
        $.ajax({
            'url': helper.apiEndpointV2('address'),
            'type': 'get',
            'dataType': 'json',
            'beforeSend': function(request) {
              helper.log("SENDING");
              helper.headers(request);
            },
            'success': function(ret) {
              if(ret.result !== 'error') {
                if(ret.data.length) {
                  helper.log(ret.data[0], "SPECIFIC");
                  storage.blob.set('address', ret.data[0]);
                  storage.set('last_address_refresh_time', now);
                }
              }
            },
            'error': function(ret) {
              helper.log('ADDRESS ERROR: ', ret);
            }
        });
    }
}
/**
 * [freshen_site_info description]
 * @param  {[type]}   data     [description]
 * @param  {Function} callback [description]
 * @return {[type]}            [description]
 */
app.freshen_site_info = function(data, callback) {
    // var last_time = storage.get('last_refresh_time'),
    //     now = new Date().getTime(),
    //     one_day = 86400000;

    // last_time = null;

    // if(last_time == null) {
    //     last_time = now - (one_day * 2);
    //     storage.set('last_refresh_time', last_time);
    // }

    helper.log(data, 'Data given to freshen_site_info function');
    var start = new Date();

    // if(parseInt(last_time) + one_day < now) {
        $.ajax({
            'url': helper.apiEndpoint('site_blob'),
            'dataType': 'json',
            'data': { 'site_id': storage.get('site_id') },
            'type': 'post',
            'success': function(blob) {
                if(blob.status == "success") {
                    // storage.set('last_refresh_time', now);
                    storage.blob.set('site_blob', blob);
                    callback(blob);
                    helper.log(blob, 'BLOB SUCCESS');
                }
                else {
                    app.Display("error_panel");
                    helper.log('blob failure');
                }
            },
            'error': function(error) {
                app.Display("error_panel");
                helper.log(error, 'AJAX ERROR');
            },
            'complete': function() {
                helper.log(['BLOB START: ' + start, 'BLOB END: ' + new Date()]);
            }
        });
    // }
    // else {
    //     app.do_site_setup(data);
    //     callback(data);
    // }
}

/**
 * Apply any data from the site_blob to the application, and append the stylesheets.
 *
 * @param  {[type]} data JSON object
 */
app.do_site_setup = function(data) {
    app.State.header.seed(data);
    // app.State.footer.seed(data);
    app.State.ad.seed(data);

    for(var i in data.css) {
        if(data.css[i].indexOf("mobile-app.css") < 0) {
            $("head").append('<link rel="stylesheet" class="site-css" type="text/css" href="'+data.css[i]+'">');
        }
    }

    app.override_styles(data);
    beacons.setup_beacons();
    ahoy.doVisitStuff();
};

/**
 * Assemble some override CSS to apply the site's custom colors to the app.
 *
 * @param  {[type]} data JSON blob of all the site data that we need
 * @return null - direct DOM access
 */
app.override_styles = function(data) {
    var styles = '
        header {
            background: none;
            background-color: #fff;
        }
        .ptext {
            color: {d_mobile_highlight_color} !important;
        }
        .pbackground {
            background-color: {d_mobile_highlight_color};
        }
        #statusBarColor {
            background-color: {d_mobile_header_background_color};
        }
        .headerColorBar {
            background-color: {d_mobile_header_background_color};
        }
        #question-button {
            background-color: {d_mobile_highlight_color};
            color: #fff;
        }
        .button.primary{
            background: none;
            background-color: {d_mobile_button_color};
            color: #fff;
        }
        #log-in.button.primary {
            background: none;
            background-color: #fff;
            color: {d_mobile_button_color};
        }
        #sign-up.button.darker-primary {
            color: #fff;
            background: none;
            background-color: {d_mobile_button_color};
        }
        .button.back, .button.back .arrow {
            background: none;
            background-color: {d_mobile_button_color};
            color: #fff;
        }
        .button.darker-primary {
            color: #fff;
            background: none;
            background-color: {d_mobile_button_color};
        }
        .progression li.active span {
            background-color: {d_mobile_highlight_color};
            color: #fff;
        }
        .stream-card {
            color: {d_mobile_highlight_color};
        }
        .overlay .question-modal {
            border-top-color: {d_mobile_highlight_color};
        }
        .overlay .question-points {
            background-color: {d_mobile_highlight_color};
        }
        .small-loading-spinner {
            color: {d_mobile_highlight_color};
        }
        nav .selected {
            color: {d_mobile_highlight_color};
        }

    ';

    for(var i in data.style.colors) {
        slug = data.style.colors[i];
        old  = new RegExp("{"+i+"}", "g");
        styles = styles.replace(old, slug);
    }

    $("<style type='text/css'></style>").append(styles).appendTo("head");
}

/**
 * Poll to see if a hidden div has recieved a style change, and when true
 * fire the callback. Used to make sure the remote stylesheet has loaded
 * so we don't get a flash of unstyled content
 *
 * @param  {Function} callback [description]
 * @return {[type]}            [description]
 */
app.styles_ready = function(callback, data) {
    var bob,
        $test = $("#test");

    app.loading(true);

    helper.log('Waiting for styles to load...');

    bob = setInterval(function()
    {
        helper.log('Width: ' + $test.width());

        if($test.width() <= 10)
        {
            helper.log('Styles loaded, firing callback.');

            clearInterval(bob);
            callback(data);
            $("#picker-css").remove();
        }
    }, 10);
}

/**
 * Fires when the device is ready
 * @return {[type]}
 */
app.onDeviceReady = function() {
    var site_blob  = null,
        session_id = null,
        username   = null;

    site_blob  = storage.blob.get('site_blob');
    session_id = storage.get('r_users');
    username   = storage.get('username');

    helper.beaconQueueSender();

    app.loading(true);

    // Try to checkin the user
    helper.auto_checkin();
    // Try to checkin the user once a minute
    window.checkinTimer = setInterval(function(){
        helper.auto_checkin();
    }, 60000) // 1 min

    if(navigator.network.connection.type == Connection.NONE) {
        helper.noData();
    }

    window.plugins.webviewcolor.change('#FFFFFF');

    app.View = [];
    app.View['activity']    = ko.observable();
    app.View['adjustments'] = ko.observable();
    app.View['upcoming']    = ko.observable();
    app.View['points']      = ko.observable();
    app.View['prizes']      = ko.observable();
    app.View['settings']    = ko.observable();

    storage.clear('user_details_request');
    storage.clear('upcoming');
    storage.clear('points');
    storage.clear('prizes');

    // Initialize the modal bindings
    app.State.modal = new app.ViewModels.modal();
    ko.applyBindings(app.State.modal, document.getElementById("modal-container"));

    // Initialize the generic modal bindings
    app.State.genericModal = new app.ViewModels.genericModal();

    // Initialize overLoad overlay
    app.State.overLoad = ko.observable(false);
    ko.applyBindings(app.State.overLoad, document.getElementById("overLoad"));
    ko.applyBindings(app.State.overLoad, document.getElementById("overSpinner"));

    // Initialize the question bindings
    app.State.survey = new app.ViewModels.survey();

    // Initialize the questionbar
    app.State.questionbar = new app.ViewModels.questionbar();

    // Initialize the alertbar
    app.State.alertbar = new app.ViewModels.alertbar();

    // Initialize the userbar
    app.State.uservisible = ko.observable(false);
    app.State.user = ko.observable();

    if(storage.blob.get('user_details')){
        app.State.user(new app.ViewModels.user(storage.blob.get('user_details'), []));
        app.State.uservisible(true);
    }

    ko.applyBindings(app.State.user, document.getElementById("headerUser"));

    // Initialize the header ViewModel.
    app.State.header = new app.ViewModels.header({}, app.State);
    ko.applyBindings(app.State.header, document.getElementById("headerColorBar"));

    // Initialize the nav ViewModel.
    app.State.nav = new app.ViewModels.navtemplate({}, app.State);
    ko.applyBindings(app.State.nav, document.getElementById("fixedNav-container"));

    // Initialize the Ad ViewModel.
    app.State.ad = new app.ViewModels.advertisement({}, app.State);
    ko.applyBindings(app.State.ad, document.getElementById("footerAd"));
    // ko.applyBindings(app.State.ad, document.getElementById("kofornow"));

    // Setup the observables for the viewmodels

    if(site_blob === null)
    {
        // No site_id, we need to show the picker!
        app.Display("picker", {});
        helper.hide_splashscreen();
    }
    else if(session_id === null) {
        // Site ID but no user? Log in buddy!
        app.freshen_site_info(site_blob, function(site_blob){
            app.do_site_setup(site_blob);

            app.styles_ready(function(blob) {
                if(helper.site_disabled(site_blob)){ app.Display('disabled', site_blob); }
                else { app.Display("authenticate", blob); }

                helper.hide_splashscreen();
            }, site_blob);
        });
    }
    else
    {
        // Check if the session_id is valid, and once it is, display the upcoming page.
        app.freshen_site_info(site_blob, function(site_blob){
            app.do_site_setup(site_blob);

            app.styles_ready(function() {
                if(helper.site_disabled(site_blob)){ app.Display('disabled', site_blob); }
                else {
                    try {
                        app.Display(site_blob.home_screen);
                    }
                    catch(e) {
                        app.Display("upcoming");
                    }
                }

                helper.hide_splashscreen();
            }, site_blob);
        });
    }

    // app.stickynav();
    beacons.initialize();
}
