/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP INFORMATION HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.site_blob = function() {
    var blob = storage.get('site_blob');
    if(blob) { return $.parseJSON(blob); }
    else { return null; }
}

helper.site_id = function() {
    return storage.get('site_id');
};

helper.user_id = function() {
    return storage.get('user_id');
};

helper.signed_in = function() { return helper.user_id(); }

helper.address = function() {
    if(storage.blob.get('address')) return storage.blob.get('address');
    else return false;
}

helper.platform = function() {
    return cordova.platformId;
}

/*
 * Jeeves is intended to be a forward facing interface tool for
 * providing informtion to the user. For example: Jeeves['location']
 * is used to format help text for enabling location services on either
 * an Android or iOS device.
 */
helper.Jeeves = [];
helper.Jeeves['location'] = function() {
    app.State.overLoad(true);
    if(helper.getAppleLocationPermissionRequested() || helper.platform() == "android") {
        navigator.geolocation.getCurrentPosition(function(){
            app.State.overLoad(false);
            app.State.alertbar.display(false);
            return;
        }, formatLocationAlert, { timeout: 10000 });
    } else { formatLocationAlert(); }

    function formatLocationAlert() {
        var copy,
            request = false;

        app.State.overLoad(false);
        switch(helper.platform()) {
            case 'ios':
                if(helper.getAppleLocationPermissionRequested()) {
                    copy = '<p>In order to earn points for attendance we need access to your location.
                     To turn on location services visit the Settings app, and navigate to Privacy / Location Services.</p><p>Make sure location services are turned on
                     then scroll the list to find this app. Tap, and change the selection to "Always" or "While using the app".</p>';
                }
                else { request = true; }
            break;
            case 'android':
                copy = '<p>In order to earn points for attendance we need access to your location.
                 Please ensure location services are enabled on your device.</p>
                 <p>Depending on your device and its version of the Android OS, make sure all options such as Location, Location services, and Location reporting are enabled.</p>';
            break;
        }

        if(request) {
            storage.set('location_last_requested', new Date(0));
            helper.requestPermissions(false);
        }

        else {
            formatted = "<div class='text-left'>"+copy+"</div>"
            helper.alertModal(formatted, 'fa-map-marker', 'Dismiss', 'Turn on Location Services');
        }
    }
}

/*
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  APP CUSTOM FUNCTIONS
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

helper.log = function(message, title, highlight) {
    if((message && false) || (message && highlight)) {

        console.log('|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||');
        if(title) { console.log('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' + title + ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'); }

        if($.isArray(message)) {
            $.each(message, function(index, value) {
                console.log('     ', value);
            });
        }

        else { console.log('     ',  message); }

        console.log('||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||');
    }
}

helper.scrollCalc = function() {
    helper.refreshCalc();
    helper.navScrollCalc();
}

helper.reloadView = false;
helper.nextPageRequestedAt = null;
helper.refreshFontSize = ko.observable(2);

helper.refreshCalc = function() {
    if(app.State.hasOwnProperty('body') && app.State.body.pullRefresh && !app.State.overLoad()) {
        if(window.myScroll.y == 0 && window.myScroll.moved && window.myScroll.distY > 0 && window.myScroll.absStartY == 0) {
            offset = window.myScroll.distY;

            if(offset > 85) {
                helper.reloadView = true;
                app.State.body.refreshIcon('fa-refresh fa-spin');
                $('.pullDownIcon').css({ 'top': '-10px' });
                helper.refreshFontSize(25);
            }

            if(offset < 640){
                if(helper.refreshFontSize() < 25) {
                    helper.refreshFontSize(helper.refreshFontSize() + 2);
                }

                app.State.body.refreshOffset(offset+'px');

            }
        }
    }

    if(app.State.hasOwnProperty('body') && app.State.body.scrollLoad && !app.State.overLoad()) {
        if(window.myScroll.absStartY == window.myScroll.maxScrollY && window.myScroll.moved && window.myScroll.distY < 0 && (helper.nextPageRequestedAt == null || helper.timeDifference(helper.nextPageRequestedAt, new Date(), 'seconds') >= 2)){
            helper.nextPageRequestedAt = new Date();
            app.State.overLoad(true);
            app.State.body.nextPage();
        }
    }
}

helper.refreshView = function() {
    view = app.State.body.template;
    helper.reloadView = false;
    storage.clear(view+'_blob');

    if(view == "activity") {
        storage.clear('adjustments_blob');
    }

    app.Display(app.State.body.template);
}

helper.navScrollCalc = function() {
    var ftb = $('.full-topbar');
    var hcb = $('#headerColorBar');
    var hcbbottom = hcb.height();
    var nav = $('#nav');
    var navOff = nav.offset().top;
    var container = $('#fixedNav-container');
    var sbc = $('#statusBarColor');
    var overspinner = $("#overSpinner");

    if(app.State.hasOwnProperty('header') && app.State.header.visible()) {
        var offset = helper.platform() === "ios" ? 20 : 0;

        if(navOff <= offset && window.myScroll.y < -12) {
            nav.css({"visibility":"hidden"});

            container.css({"z-index":"1", "visibility": "visible"});
            navBottom = 0;
            overspinner.css({ "top":navBottom, "bottom":"0" });
        }
        else {
            nav.css({"visibility":"visible"});
            container.css({"z-index":"0", "visibility":"hidden"});
            overspinner.css({ "top":navOff, "bottom":"0" });
        }

        if(helper.platform() === "ios" && hcb.is(':visible')) {

            if((hcb.offset().top+hcbbottom) <= 0) {
                sbc.fadeIn();
            }
            else {
                sbc.fadeOut();
            }
        }
        else if(helper.platform() === "ios" && ftb.is(':visible')) {

            if(ftb.offset().top <= -10) {
                sbc.fadeIn();
            }
            else {
                sbc.fadeOut();
            }
        }
    }
}

helper.refreshScroll = function(){
    myScroll.refresh();
    helper.scrollCalc();
}


helper.beaconQueueSender = function(){
    if(typeof beacons !== "undefined"
    && beacons !== null
    && storage.blob.get('beaconQueue') !== null
    && storage.blob.get('beaconQueue').length > 1)
    {
        beacons.send_range_action(storage.blob.get('beaconQueue'));
    }
}


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP BOOLEAN HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.isInArray = function(value, array) {
    return array.indexOf(value) > -1;
}

helper.crossedThreshold = function(cost){
    return app.State.user().lifetime_points() && app.State.user().lifetime_points().toString().replace(/,/g , "") >= cost;
}


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP CONNECTION SETTINGS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/**
 * Visual handler for when no data connection is available.
 */
helper.noData = function() {
    var $dataMask = $('#data-mask');

    $('body').addClass('blur');
    $dataMask.show();

    if(typeof noDataCallback !== "undefined") {
        navigator.notification.confirm(
            'Network Required',
            noDataCallback,
            'A network connection is required',
            ['Ok','Cancel']
        );
    }
};

/**
 * Hide the data mask when data access is available
 */
helper.hasData = function() {
    var $dataMask = $('#data-mask');

    $('body').removeClass('blur');
    $dataMask.hide();
};


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP SETTINGS/SWITCH HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.site_disabled = function(data) {
  return typeof data.site_enabled == 'undefined' ? false : !data.site_enabled;
}

// Can only be called on the prize page
helper.toggleCheckout = function() {
    app.State.body.checkoutVisible(!app.State.body.checkoutVisible());
    app.State.modal.active(false);
}

helper.isLoading = ko.observable(false);
helper.adAvailable = ko.observable(false);
helper.display_avatar = ko.observable(false);
helper.facebookOauthLink = ko.observable('');
helper.facebookConnected = ko.observable(false);


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP STORAGE HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

//http://stackoverflow.com/questions/4391575/how-to-find-the-size-of-localstorage
helper.check_storage = function() {
    var all = 0; 
    for(var x in localStorage){
        helper.log(x+"="+((localStorage[x].length * 2)/1024/1024).toFixed(2)+" MB");
        all = all + parseFloat(((localStorage[x].length * 2)/1024/1024).toFixed(2));
    }

    helper.log("Total storage used: "+ all +" MB");
}

helper.check_queue_size = function() {
    return ((storage.blob.get('beaconQueue').length * 2)/1024).toFixed(2);
}


helper.set_view = function(type, blob) { storage.set(type+'_blob', JSON.stringify(blob)); }
helper.get_view = function(type) { return JSON.parse(storage.get(type+'_blob')); }


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP DATA MANIPULATION HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.punctuate_number = function(n) {
    //http://stackoverflow.com/questions/2901102/how-to-print-a-number-with-commas-as-thousands-separators-in-javascript

    return n.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}

helper.timeDifference = function(date1, date2, to){
    to = to ? to : 'minutes';
    d1 = new Date(date1);
    d2 = new Date(date2);

    diff = d1 >= d2 ? d1-d2 : d2-d1;

    switch(to) {
        case 'seconds':
            return diff/1000;
            break;
        case "minutes":
            return diff/1000/60;
            break;
        case "hours":
            return diff/1000/60/60;
            break;
        case "days":
            return diff/1000/60/60/24;
            break;
    }
}

helper.compareTime = function(date1, date2) {
    d1 = new Date(date1);
    d2 = new Date(date2);

    return d1 > d2;
}

helper.timestamp = function(seconds){
    d = new Date();
    return time = ""+d.getFullYear()+(d.getMonth()+1)+d.getDate()+d.getHours()+d.getMinutes()+helper.round(d.getSeconds(), seconds)
}

helper.round = function(n, v) {
    v = v ? v : 10;
    return Math.round(n / v) * v
}

/**
* We are assuming that if we have made it to here that d is a valid date
* v (value) the requested format. EX: v = j means that we want the the day of
* month from 1-31
* */
helper.getFormat = function(d, v) {
    t = [];

    fullM = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
    abvM  = ["Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"];
    fullD = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
    abvD  = ["Sun", "Mon", "Tue", "Wed", "Thr", "Fri", "Sat"];

    // YEAR
    t['Y'] = d.getFullYear(); // 2015
    t['y'] = (d.getFullYear()+"").substr(2,5); // 15

    // MONTH
    t['n'] = d.getMonth()+1; // 1-12
    t['m'] = t['n'] < 10 ? "0"+t['n'] : t['n'] // 01-12
    t['F'] = fullM[d.getMonth()]; // January - December
    t['M'] = abvM[d.getMonth()]; // Jan - Dec

    // DAY
    t['j'] = d.getDate(); // 1-31
    t['d'] = t['j'] < 10 ? "0" + t['j'] : t['j']; // 01-31
    t['l'] = fullD[d.getDay()]; // Sunday - Saturday
    t['D'] = abvD[d.getDay()]; // Sun - Sat

    // HOUR
    t['G'] = d.getHours() // 0-23
    t['g'] = (t['G'] % 12) ? (t['G'] % 12) : 12 // 1-12
    t['H'] = t['G'] < 10 ? "0"+t['G'] : t['G']; // 00-23
    t['h'] = t['g'] < 10 ? "0"+t['g'] : t['g']; // 01-12

    // TIME OF DAY
    t['a'] = t['G'] < 12 ? 'am' : 'pm';
    t['A'] = t['G'] < 12 ? 'AM' : 'PM';

    // MINUTES
    t['i'] = d.getMinutes() < 10 ? "0"+d.getMinutes() : d.getMinutes(); // 00-59

    // SECONDS
    t['s'] = d.getSeconds() < 10 ? "0"+d.getSeconds() : d.getSeconds(); // 00-59

    return (t[v] == 'undefined' || t[v] == undefined) ? v : t[v];
}

/*
* t = a time stamp
* format = 'Y-m-d' etc
*/
helper.displayTime = function(t, format) {
    d = new Date(t);
    f = format.split("");
    time = "";

    if(d !== "Invalid Date") {
        $.each(f, function(i, v){
            time += helper.getFormat(d, v)
        });
    }

    else { time = false; }

    return time;
}

/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  AUTO GPS CHECKIN
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

  var ob = {};
      ob.events = null;

  ob.postCheckin = function(location){

    helper.log('CHECKING IN AJAX');
    params = { 'latitude': location.coords.latitude, 'longitude': location.coords.longitude };
    helper.log(params);

    stringifed = JSON.stringify($.extend(params, app.defaultAjaxParams()));
    $.ajax({
      beforeSend: function (request){ helper.headers(request, stringifed); },
      url       : helper.apiEndpointV2('auto_gps_checkin'),
      method    : "POST",
      data      : params,
      contentType: "application/json",
      success   : function(result) {
          if (result.status == 200) {
            helper.log(result, 'AUTO CHECKIN RESPONSE');
            ob.checkinSuccess(result);
          }

          helper.refreshScroll();
      },
      error    : function(e) {
        helper.log(e, "ERROR");
      }
    });
  }

  ob.noGps = function() {
    if(session_id = storage.get('r_users') && typeof app.State.body !== "undefined" && typeof app.State.body !== undefined){
        if(app.State.body.template == "upcoming") {
            app.State.alertbar.text('Please turn on Location Services to earn Attendance Points <i class="fa fa-info-circle"></i>');
            app.State.alertbar.click(helper.Jeeves['location']);
            app.State.alertbar.display(true);

            $.each(ob.events, function(){
                var self = this;
                self.unfindable(true);
            });
        }
    }

    // ob.checkinError('Please turn on Location Services to earn Attendance Points <i class="fa fa-info-circle"></i>');
  }

  ob.checkinError = function(error) {
      if(app.State.body.template == "upcoming") {
        app.State.alertbar.text(error);
        app.State.alertbar.display(true);
      }
  }

  ob.checkinSuccess = function(obj) {
    var message = "";
    if(obj.data.length > 0) {
        $.each(obj.data, function(key, value){
            helper.log(value.event, "THE EVENT");
            if(value.result.success) {
                if(value.result.attendee_message || value.result.message) {
                    message += value.result.attendee_message ? value.result.attendee_message : value.result.message;
                    message += "<br>";
                }

                helper.log(ob.events[value.result.event_id], "THE SUCCESSFUL EVENT");
                var self = this;

                ob.events[value.event.id].checked_in(true);
                ob.events[value.event.id].update_variables(value.event.user_checkin_points);
            }
        });
    }

    if(message !== "") {
        helper.log(message, 'This is the message');
        if(!app.State.modal.active()) {
            helper.alertModal(message, 'fa-map-marker');
        }
        else {
            checkinMessageTimer = setTimeout(function(){
                if(!app.State.modal.active()) {
                    clearInterval(checkinMessageTimer);
                    helper.alertModal(message, 'fa-map-marker');
                }
            }, 3000);
        }
    }
  }

  helper.checkin = function() {
    helper.log('CHECKIN IN FUNCTION CALLED')
    if(navigator.geolocation) {
        cordova.plugins.locationManager.requestWhenInUseAuthorization();
        navigator.geolocation.getCurrentPosition(ob.postCheckin, ob.noGps, { timeout: 10000 });
    } else {
        // ob.checkinError('Please turn on Location Services to earn Attendance Points <i class="fa fa-info-circle"></i>');
    }

    helper.refreshScroll();
  }

  helper.auto_checkin = function(events) {
    if(events){
        e = {};
        $.each(events, function(){
            e[this.id] = this;
        });

        ob.events = e;
    }

    helper.log(ob.events, "AUTO CHECKIN");
    if(helper.platform() !== "ios" || helper.getAppleLocationPermissionRequested()) {
        helper.checkin();
    }
    else { ob.noGps(); }
  }

/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  MANUAL GPS CHECKIN
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

var mc = {};
    mc.event_id = null;
    mc.event = null;

helper.manual_checkin = function(event_id, event) {
    helper.log("MANUAL CHECKIN", "MANUAL CHECKIN");

    mc.event = event;
    mc.event_id = event_id;

    helper.log([helper.platform() !== "ios", helper.getAppleLocationPermissionRequested()], 'MANUAL CHECKIN TEST');

    if(helper.platform() !== "ios" || helper.getAppleLocationPermissionRequested()) {
      mc.checkin();
    }
    else { mc.noGps(); }
}

mc.checkin = function() {
    if(navigator.geolocation) {
        cordova.plugins.locationManager.requestWhenInUseAuthorization();
        navigator.geolocation.getCurrentPosition(mc.postCheckin, mc.noGps, { timeout: 10000 });
    } else {
        mc.noGps();
    }

    helper.refreshScroll();
}

mc.noGps = function(err) {
    if(err) {
        helper.log('ERROR('+err.code+') '+ err.message, 'NO GPS');
    }

    if(mc.event) {
        mc.event.checkingin(false);
        mc.event.unfindable(true);
        mc.event.colorize();
    }
}

mc.postCheckin = function(location){
    if(mc.event_id){
        helper.log('MANUAL CHECKIN AJAX');

        params = { 'latitude': location.coords.latitude, 'longitude': location.coords.longitude };
        helper.log(params, 'DES PARAMS');

        stringifed = JSON.stringify($.extend(params, app.defaultAjaxParams()));
        $.ajax({
          beforeSend: function (request){ helper.headers(request, stringifed); },
          url       : helper.apiEndpointV2('manual_gps_checkin', mc.event_id),
          method    : "POST",
          data      : params,
          contentType: "application/json",
          success   : function(result) {
            if (result.status == 200) {
                helper.log(result, 'MANUAL CHECKIN RESPONSE');
                mc.checkinSuccess(result.data);
            }
            else {
                if(mc.event){
                    mc.event.checkingin(false);
                    // result.message
                }
            }

            mc.event_id = null;

            helper.refreshScroll();
          },
          error    : function(e) {
            helper.log(e, "ERROR");
            mc.event_id = null;
          }
        });
    }
    else {
        helper.log(mc.event_id, 'EVENT ID ERROR');
        mc.event_id = null;
    }
}

mc.checkinSuccess = function(obj) {
    helper.log(obj, 'CHECKIN SUCCESS')
    if(mc.event){
        mc.event.checkingin(false);

        if(obj.result.success) {
            mc.event.checked_in(obj.event.checked_in);
            mc.event.update_variables(obj.event);
        }
        else {
            if(obj.result.message == 'You are too far away from the venue to check in.')
                mc.event.distance(true);

            if(obj.result.message !== "You have already been checked in to this event.")
                mc.event.colorize();

            else
                mc.event.update_variables(obj.event);
        }
    }
}

/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  PUSH MODAL FUNCTIONS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.submit = [];
helper.skip = [];

helper.skipPushModal = function() {
    blob = helper.site_blob();
    return blob.skipPushModal;
}

/*
* Check if we have had the Apple Push Notification Permissions modal called yet.
*/
helper.getApplePushPermissionRequested = function() {
    return storage.get('push_requested') ? storage.get('push_requested') : false;
}

/*
* This is what will pop the Apple Push Notification Permissions modal.
* It also sets a variable to make sure we don't try to ask for
* Push Notification permissions again. We only get 1 shot at this.
*/
helper.submit['push'] = function() {
    storage.set('push_requested', true);
    helper.push.init();
    app.State.modal.active(false);

    if(helper.permissionsRequestRequired()) {
        helper.requestPermissions();
    }
}

/*
* Get the variable in storage that check the last time we asked for push permissions
*/
helper.getPushLastRequested = function() {
    return storage.get('push_last_requested') ? storage.get('push_last_requested') : false;
}
/*
* Sets a variable in storage to check the time we last asked for push permissions
*/
helper.skip['push'] = function() {
    var date = new Date();

    delay = helper.delayAmount['push'] ? helper.delayAmount['push'] : 7;
    type = helper.delayType['push'] ? helper.delayType['push'] : 'date';

    d = helper.setDelayPeriod(delay, type);

    storage.set('push_last_requested', d);
    app.State.modal.active(false);

    if(helper.permissionsRequestRequired()) {
        helper.requestPermissions();
    }
}



/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  LOCATION MODAL FUNCTIONS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/*
* Check if we have had the Apple Location Permissions modal called yet.
*/
helper.getAppleLocationPermissionRequested = function() {
    return storage.get('location_requested') ? storage.get('location_requested') : false;
}

/*
* This is what will pop the Apple Location Permissions modal.
* It also sets a variable to make sure we don't try to ask for
* location permissions again. We only get 1 shot at this.
*/
helper.submit['location'] = function() {
    cordova.plugins.locationManager.requestAlwaysAuthorization();
    storage.set('location_requested', true);
    app.State.modal.active(false);

    if(helper.permissionsRequestRequired()) {
        helper.requestPermissions();
    }
}

/*
* Get the variable in storage that check the last time we asked for location permissions
*/
helper.getLocationLastRequested = function() {
    return storage.get('location_last_requested') ? storage.get('location_last_requested') : false;
}
/*
* Sets a variable in storage to check the time we last asked for location permissions
*/
helper.skip['location'] = function() {
    var date = new Date();

    delay = helper.delayAmount['location'] ? helper.delayAmount['location'] : 7;
    type = helper.delayType['location'] ? helper.delayType['location'] : 'date';

    d = helper.setDelayPeriod(delay, type);

    storage.set('location_last_requested', d);
    app.State.modal.active(false);

    if(helper.permissionsRequestRequired()) {
        helper.requestPermissions();
    }
}


/*
* This sets up the callback and the time reference functions that will be
* used for the various portions of the modal.
*/
helper.permissionsModalSettings = function(type) {
    var settings = {};
    var self = this;

    type = type.toLowerCase();

    self.options = [];

    self.options['push'] = function() {
        settings = {
            func : 'push',

            lastRequested  : helper.getPushLastRequested(),
            appleRequested : helper.getApplePushPermissionRequested()
        }
    }

    self.options['location'] = function() {
        settings = {
            func : 'location',

            lastRequested  : helper.getLocationLastRequested(),
            appleRequested : helper.getAppleLocationPermissionRequested()
        }
    }

    if(typeof self.options[type] !== "undefined"){
        self.options[type]();
    }

    return settings;
}

helper.permissionsRequestRequired = function() {
    helper.log('permissionsRequestRequired');

    if(cordova.platformId !== "ios") {
        return false;
    }

    app.permissionsAsk = false;

    response = false;

    blob = helper.site_blob();

    $.each(blob.permission_modals, function(){
        var self = this;

        if(helper.askPermission(self.type)) {
            response = true;
        }
    });

    app.permissionsAsk = true;

    return response;
}

helper.delayAmount = [];
helper.delayType = [];

helper.requestPermissions = function(delay) {
    delay = (delay == false) ? delay : true;
    app.permissionsAsk = false;

    blob = helper.site_blob();

    $.each(blob.permission_modals, function(){
        var self = this;

        amount = (!delay || self.SkipInitialDelay) ? 0 : self.InitialDelay;

        if(self.type == 'push' && helper.skipPushModal()) { return true; }

        helper.delayAmount[self.type] = self.DelayAmount;
        helper.delayType[self.type] = self.DelayType;
        if(helper.askPermission(self.type)) {
            setTimeout(function(){
                helper.log("Imma ask "+ self.type +" now");

                helper.PermissionModal(self);
            }, amount);
        }
    });

    app.permissionsAsk = true;
}

helper.askPermission = function(type) {
    var settings = helper.permissionsModalSettings(type);
    var d = new Date();

    if(type == "push" && helper.skipPushModal()) { return false; }

    if(settings.lastRequested == false && settings.appleRequested == false) { ask = true; }
    else if(settings.lastRequested !== null && typeof settings.lastRequested !== "undefined" && settings.appleRequested == false) {
        var date = new Date(settings.lastRequested);
        date.setMinutes(date.getMinutes());
        ask = date <= d;
    }
    else { ask = false; }

    helper.log('This is what I determined ask is for ' + type + ": " + ask);

    return ask;
}

helper.setDelayPeriod = function(amount, type) {
    d = new Date();

    switch(type) {
        case "seconds":
            d = new Date(d.getTime() + parseInt(amount)*1000)
            break;
        case "minutes":
            d = new Date(d.getTime() + parseInt(amount)*60000)
            break;
        case "date":
        case "days":
            d.setDate(d.getDate()+parseInt(amount));
            break;
        case "year":
            d.setFullYear(d.getFullYear() + parseInt(amount));
            break;
    }

    return d;
}

/*
* This initializes our modal that asks for permissions BEFORE the Apple's.
* The permissions modal of Apple's won't pop unless the user expresses
* interest in consenting to having the app use whatever permission is needed.
* Since we only get 1 shot to ask for permissions, lets try to get
* a favorable response. This way we get the option to ask again later.
*/
helper.PermissionModal = function(data) {
    var d = new Date();

    var settings = helper.permissionsModalSettings(data.type);

    helper.log(settings, 'settings');

    // Settings for the location modal are editable in the rewards platform in the mobile/api.php
    if(!settings.appleRequested && !app.State.modal.active() && data) {
        if(settings) {
            app.State.modal.template('genericModal');
            app.State.modal.content(app.State.genericModal);

            app.State.genericModal.icon(data.Icon);
            app.State.genericModal.text(data.Title);
            app.State.genericModal.wellText(data.WellText);

            app.State.genericModal.func(settings.func);

            app.State.genericModal.submit_button_text(data.SubmitButtonText || "Allow");
            app.State.genericModal.skip_button_text(data.SkipButtonText || "Not Now");
            app.State.genericModal.submit_additional_class('');

            app.State.modal.active(true);
        }
    }
}

helper.checkPermissions = function() {
    // Permissions would be useful. Lets see if we have them or ask for them if it seems appropriate...
    if(app.permissionsAsk && helper.signed_in()){
        if(helper.permissionsRequestRequired()) {
            helper.requestPermissions();
        }
    }

    // We need to register parse on android devices and ios devices
    // that have already seen the apple request for notifications.
    if(window.device.platform !== "iOS"
        || helper.getApplePushPermissionRequested()
        || helper.skipPushModal()) { helper.push.init(); }
}

/*
* This preps our modal to display an alert message that can be dismissed.
* It accepts:
*     • a requred message to display
*     • an optional font-awesome icon to use
*     • an optional overide of the dismiss button's text
*     • an optional title
*     • an optional overide to the submit funcion
*     • an optional overide of the skip text
*     • an optional overide of the skip function.
*/
helper.alertModal = function(message, icon, button_text, title, submitFunc, skip_button_text, skipFunc) {
    if(!app.State.modal.active()){
        var icon = icon && icon !== "default" ? icon : 'fa-trophy';
        var title = title ? title : '';
        var button_text = button_text ? button_text : 'Dismiss';

        app.State.genericModal.icon(icon);

        app.State.genericModal.text(title);
        app.State.genericModal.wellText(message);
        app.State.genericModal.submit_additional_class('');

        if(submitFunc){
            app.State.genericModal.func(submitFunc);
            if(!skipFunc) { app.State.genericModal.submit_additional_class('clearFloat'); }
        }
        else {
            app.State.genericModal.func(function(){ app.State.modal.active(false); });
            app.State.genericModal.submit_additional_class('clearFloat');
        }

        if(skipFunc) {
            app.State.genericModal.skipFunc(skipFunc);
        }
        else if(submitFunc) {
            app.State.genericModal.skipFunc(function(){ app.State.modal.active(false); });
        }

        app.State.genericModal.submit_button_text(button_text);
        app.State.genericModal.skip_button_text( skip_button_text ? skip_button_text : '');

        app.State.modal.template('genericModal');
        app.State.modal.content(app.State.genericModal);

        app.State.modal.active(true);
    }
}

helper.confirmModal = function(message, icon, accept_text, reject_text, success_callback){
    helper.alertModal(message, icon, accept_text, '', function(){ app.State.modal.active(false); success_callback(); }, reject_text, function(){ app.State.modal.active(false); });
}


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP ALERT FUNCTIONS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/*
* Used to call a small bar for alert messages
*/
helper.alertbar = function(message, style, timeout) {
    style = style ? style : "error";

    app.State.alertbar.type(style == "success" ? "alert-success" : "alert-error");
    app.State.alertbar.text(message);
    app.State.alertbar.display(true);

    if(timeout) {
        setTimeout(function(){
            $('.alertbar').slideToggle('fast', function(){
                app.State.alertbar.display(false);
            });
        },timeout);
    }
}

/*
* This displays an error message inside the any modal.
*/
helper.warnInModal = function(warning, hold) {
    clearTimeout(timer);

    if(app.State.modal.warning() !== warning) {
        app.State.modal.warning(warning);

        if(!$('.modal-warning').is(':visible')) {
            $('.modal-warning').slideToggle();
        }
    }

    if(!hold) {
        var timer = setTimeout(function() {
            $('.modal-warning').slideToggle('fast', function(){
                app.State.modal.warning('');
            });
        }, 5000);
    }
}


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP QUESTION HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/**
* This will ask the API if there is a new question to ask the user.
*/
helper.checkQuestion = function() {
    helper.log('checkQuestion');
  $.ajax({
    beforeSend: function (request){ helper.headers(request); },
    url: helper.apiEndpointV1("questions"),
    success: function(result) {
      if(result.success){
        helper.log(result);
        if(result.data.question) {
            app.State.modal.content(app.State.survey);
            app.State.modal.template('survey');
            app.State.survey.updateContent(result);
        }
      }
      else {
        helper.log('failure');
      }
    }
  });
}




/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP API HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

/*
* This is comes down from platform and represents a user's signing_key on the users table
* We use crypto to make a SHA256 hash to authorize API interactions.
*/
helper.auth = function(content){
    content = typeof content !== 'undefined' ?  content : "";
    if(storage.get("signing_key")){
        var hash = CryptoJS.HmacSHA256(content, storage.get("signing_key"));
        return hash.toString(CryptoJS.enc.Hex);
    }
}

/**
* These are the headers that are sent before API interaction that authorizes our user.
*/
helper.headers = function(request, body){
    body = body ? body : "";
    request.setRequestHeader("X-FanMaker-Auth", helper.auth(body));
    request.setRequestHeader("X-FanMaker-User-Id", helper.user_id());
    request.setRequestHeader("X-FanMaker-Site-Id", helper.site_id());

    helper.log(helper.auth(body));

    return request;
}


helper.apiEndpointV1 = function(name, item_id) {
    // var base = 'http://api.rewards-admin.dev/api/v1/',
    var base = 'https://api.fanmaker.com/api/v1/',
        endpoint = '';

    helper.log([base, name], 'API V1');

    switch(name) {
        case 'beacon_regions':
            endpoint = 'sites/' + helper.site_id() + '/beacon_regions.json';
            break;
        case 'beacon_region_action':
            endpoint = 'sites/' + helper.site_id() + '/beacon_regions/' + item_id + '/beacon_region_actions.json';
            break;
        case 'beacon_range_action':
            endpoint = 'sites/' + helper.site_id() + '/beacon_range_actions.json';
            break;
        case 'gps_checkin':
            endpoint = 'events/' + item_id + '/gps_checkins';
            break;
        case 'questions':
            endpoint = 'questions/';
            break;
        case 'question_response':
            endpoint = 'questions/' + item_id + '/question_response';
            break;
        case 'question_update':
            endpoint = 'questions/' + item_id;
            break;
        case 'bid_history':
            endpoint = "auctions/" + item_id;
            break;
        case 'submit_bid':
            endpoint = 'auctions/' + item_id + '/max_bid';
            break;
        case 'shipping':
            endpoint = 'shipping';
            break;
    }

    return base + endpoint;
};

helper.apiEndpointV2 = function(name, item_id) {
    // var base = 'http://api.rewards-admin.dev/api/v2/',
    var base = 'https://api.fanmaker.com/api/v2/',
        endpoint = '';

    helper.log([base, name], 'API V2');

    switch(name) {
        case 'upcoming':
            endpoint = 'events';
            break;
        case 'points':
            endpoint = 'site/scorables';
            break;
        case 'prizes':
            endpoint = 'store';
            break;
        case 'redeem_contest':
            endpoint = 'store/contests/' + item_id + '/enter';
            break;
        case 'redeem_item':
            endpoint = 'store/items/' + item_id + '/redeem';
            break;
        case 'activity':
            endpoint = 'activities';
            break;
        case 'adjustments':
            endpoint = 'adjustments';
            break;
        case 'auto_gps_checkin':
            endpoint = 'events/auto_checkin';
            break;
        case 'manual_gps_checkin':
            endpoint = 'events/' + item_id +'/checkin';
            break;
        case 'address':
            endpoint = 'addresses';
            break;
    }

    return base + endpoint;
};

/**
 *  Keep all the API endpoints in one function for easier changing
 *  Match a static english name to a fully qualified URL
 *
 *  @param name = english name of the endpoint, shouldn't change
 *  @returns string
 */
helper.apiEndpoint = function(name, item_id) {
    // var base = 'http://demo.fanmaker.dev/',
    var base = 'http://mobile.fanmaker.com/',
        endpoint = '';

    switch(name) {
        case 'ahoy_event':
            endpoint = 'mobile/api/ahoy_event';
            break;
        case 'ahoy_visit':
            endpoint = 'mobile/api/ahoy_visit';
            break;
        case 'user_details':
            endpoint = 'mobile/api/user_details';
            break;
        case 'sites_list':
            endpoint = 'api/sites/mobile';
            break;
        case 'site_blob':
        	endpoint = 'mobile/api/site_blob';
        	break;
        case 'reset':
            endpoint = 'mobile/api/reset';
            break;
        case 'login_form':
        	endpoint = 'mobile/api/login';
        	break;
        // events_list
        case 'upcoming':
            endpoint = 'mobile/api/events_list';
            break;
        // points_list
        case 'points':
            endpoint = 'mobile/api/points_list';
            break;
        // prizes_list
        case 'prizes':
            endpoint = 'mobile/prizes/json';
            break;
        case 'registration_form':
            endpoint = 'mobile/api/registration_form';
            break;
        case 'register':
            endpoint = 'mobile/api/register';
            break;
        case 'settings':
            endpoint = 'mobile/api/settings';
            break;
        case 'checkin':
            endpoint = 'mobile/api/checkin';
            break;
        case 'logout':
            endpoint = 'mobile/api/logout';
            break;
    }

    return base + endpoint;
};

// Retreive and set user details
helper.getUserDetails = function(override) {
    var last_time = storage.get('user_info_requested'),
        now = new Date().getTime(),
        one_week = 86400000*7;

    if(override) { last_time = null; }

    if(last_time == null || isNaN(last_time)) {
        last_time = now - (one_week * 2);
        storage.set('user_info_requested', last_time);
    }

    if(parseInt(last_time) + one_week < now) {
        var start = new Date();
        $.ajax({
            'url': helper.apiEndpoint('user_details'),
            'type': 'get',
            'success': function(ret) {
                helper.log(ret, 'Users Data');
                if(ret.result === "success") {
                    if(ret.user_details) {
                        helper.updateUser(ret.user_details);
                        storage.set('last_address_refresh_time', now);
                    }
                } else {
                    app.loading(false);
                }

            },
            'error': function(ret) {
                helper.log(ret);
            },
            'complete': function() {
                helper.log(['USER START: ', start, ' USER END: ', new Date()]);
            }
        });
    }
}

helper.updateUser = function(user) {
    storage.blob.set('user_details', user);
    app.State.user(new app.ViewModels.user(user, []));
    app.State.uservisible(true);
}


// Ajax calls for view details
helper.requestProcessing = function(requestType, blob) {
    blob = blob ? blob : false;
    if(blob) {
        if(storage.blob.get(requestType) && !storage.blob.get(requestType).hasOwnProperty('date')) { storage.clear(requestType); }

        if(storage.blob.get(requestType) && storage.blob.get(requestType).hasOwnProperty('date')
           && (new Date(storage.blob.get(requestType).date.getTime() + 10000)) <= new Date()) {
            storage.clear(requestType);
        }

        return storage.blob.get(requestType) == null ? "false" : storage.blob.get(requestType);
    }
    else {
        if(storage.get(requestType) && !storage.get(requestType).hasOwnProperty('date')) { storage.clear(requestType); }

        if(storage.get(requestType) && storage.get(requestType).hasOwnProperty('date')
           && (new Date(storage.get(requestType).date.getTime() + 10000)) <= new Date()) {
            storage.clear(requestType);
        }

        return storage.get(requestType) == null ? "false" : storage.get(requestType);
    }
}

helper.setRequest = function(requestType, value, date) {
    storage.set(requestType, { 'value': value, 'date': date });
}

helper.api = [];
helper.api['platform'] = helper.apiEndpoint;
helper.api['admin1'] = helper.apiEndpointV1;
helper.api['admin2'] = helper.apiEndpointV2;


helper.requestData = function(requestType, overlay, params, method, v, skipStorage) {
    skipStorage = skipStorage ? skipStorage : false;
    overlay = overlay ? overlay : false;
    method = method ? method : 'platform';
    v = v ? v : 1;
    var func = '';
    // ALL REQUESTS

    helper.log(requestType, 'REQUESTED DATA');

    switch(method) {
        case "admin":
            func = 'admin'+v;
            break;
        default:
            func = 'platform';
            break;
    }

    helper.log(helper.requestProcessing(requestType), 'REQUEST PROCESSING?');

    if(helper.requestProcessing(requestType) == "false"
        || helper.requestProcessing(requestType) == false
        || skipStorage) {
        // app.smallLoad(true);
        var start = new Date();

        helper.log(helper.api[func](requestType), 'URL');

        $.ajax({
          'url': helper.api[func](requestType),
          'type': 'get',
          'data': params,
          'dataType': 'json',
          'beforeSend': function(request) {
            if(method == 'admin') { helper.headers(request); }

            helper.setRequest(requestType, true, new Date());
            if(overlay && !helper.isLoading()) { app.State.overLoad(true); }
          },
          'success': function(ret) {
            if(ret.result !== 'error') {
              helper.log([requestType + ' result: ', ret], 'REQUEST SUCCESS');

                if(!skipStorage){
                    helper.set_view(requestType, ret);
                }

              app.View[requestType](ret);

              helper.log(ret, 'UPDATED VIEW');

              // helper.facebookConnected(ret.user.fb_connected);
              // helper.facebookOauthLink(ret.user.fb_oauth_path);
            }
          },
          'error': function(ret) {
            helper.log([requestType +": ERROR WHILE FETCHING DATA: ", ret], 'REQUEST ERROR');
            helper.setRequest(requestType, false, new Date());
            app.Display("error_panel");
          },
          'complete': function() {
            app.smallLoad(false);
            helper.setRequest(requestType, false, new Date());
            helper.log([requestType + ' START: ', start, requestType + ' END: ', new Date()], 'REQUEST COMPLETE');
            app.State.overLoad(false);
          }
        });
    }
}


/*
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*  APP BEHAVIOR HELPERS
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

helper.resume = function() {

    try { app.Display(app.State.body.template); }
    catch(e) { app.Display('upcoming'); }

    beacons.setup_beacons();
    helper.beaconQueueSender();
    clearInterval(window.checkinTimer);

    window.checkinTimer = setInterval(function(){
        helper.auto_checkin();
    }, 60000) // 1 min
}

helper.pause = function(){
    clearInterval(window.checkinTimer);

    window.checkinTimer = setInterval(function(){
        helper.auto_checkin();
    }, 60000*15) // 15 min
}

/**
 * Log the user in
 * @param  {[type]} session_id r_users slug
 */
helper.log_in = function(session_id, username, user_id, signing_key) {
    storage.set('r_users', session_id);
    storage.set('username', username);
    storage.set('user_id', user_id);
    storage.set('signing_key', signing_key);
};

/**
 * Check if we have all the right data.
 * Otherwise we sign the user out.
 */
helper.userMissingData = function(){
    if(!storage.get('r_users') || !storage.get('username') || !storage.get('user_id')
        || storage.get('user_id') === "undefined" || !storage.get('signing_key')
        || storage.get('signing_key') === "undefined"){

        // alert('Sorry, somthing went wrong. Please sign in again.');
        helper.sign_out();
        return;
    }
}

/**
 * Sign the user out
 */
helper.sign_out = function() {
    $.ajax({
        url: helper.apiEndpoint("logout"),
        type: 'post'
    });

    helper.clear_data();

    window.location.reload();
}

/**
 * Log the user out
 */
helper.log_out = function() {
    storage.clear('r_users');
    storage.clear('username');
    storage.clear('user_id');
    beacons.unregister_regions();

    storage.clear('ahoy_visit');
    storage.clear('ahoy_visitor');
    storage.clear('ahoy_track');
    storage.clear('ahoy_expires_at');
};

/**
 * Wipe out all the data we save and reset the app
 */
helper.clear_data = function() {
    helper.log_out();

    if(app.standalone === false) {
        storage.clear('site_id');
        storage.clear('site_blob');
        storage.clear('last_refresh_time');

        // Remove site-specific CSS
        $(".site-css").remove();

        // Append the picker CSS if it's not attached.
        if($("#picker-css").length === 0) {
            $("header").append('<link rel="stylesheet" type="text/css" href="css/index.css" id="picker-css"/>');
        }
    }
};

/**
 * Return the second to last history object for re-navigating
 * Technically this is the 'last' page, since the current page is already in history
 * @return {[type]} [description]
 */
helper.last_page = function() {
    var length = app.history.length;

    return app.history[length - 2];
};
helper.this_page = function() {
    return app.history[app.history.length - 1];
};

/**
 * Centeralizing this API call in case we need ot wrap it with anything
 * @return {[type]} [description]
 */
helper.hide_splashscreen = function() {
    setTimeout(function(){
        helper.log('Hiding splash');
        navigator.splashscreen.hide();
    }, 1000);
};

/**
 * Sometimes iScroll needs to be destroyed. This will take care of that for us.
 * @return {[type]} [description]
 */
helper.kill_iscroll = function() {
    setTimeout(function(){
        window.myScroll.destroy();
        window.myScroll = null;
        $("#scroller").attr("style", "");
    }, 10);
};

/**
 * Handles the oAuth connections. Pop open a child browser, point it at the Rewards endpoint,
 * and react to the location it finally stops loading at.
 *
 * @param  {[type]} path             Oauth URI
 * @param  {[type]} $self            jQuery object of the element that was clicked
 * @param  {[type]} success_callback callback to fire on a successful connection
 * @param  {[type]} callback_data    data for the callback
 * @return {[type]}                  void
 */
helper.oauth_connection_handler = function(path, $self, success_callback, callback_data)
{
    var browser;

    browser = window.open(path, '_blank');
    browser.addEventListener('loadstop', function(event){
        // If the browser stopped on the error URL
        if(event.url.indexOf("http://mobile.fanmaker.com/mobile/connections/oauth/error/") != -1) {
            setTimeout(function(){
                browser.close();
            }, 3000);
        }
        // If the browser stopped on the success URL, auto-close
        if(event.url.indexOf("http://mobile.fanmaker.com/mobile/connections/oauth/success/") != -1) {
            browser.close();
            success_callback(callback_data);
        }
    });

    helper.getUserDetails();

    helper.refreshScroll();
};


helper.push = {};
helper.push.init = function() {
    parsePlugin.initialize(app.parse_application_id, app.parse_client_key, function(){
        helper.push.channel_subscribe();
    },
    function(e){
       helper.log(e, 'Parse Error');
    });
    helper.log('push init');
};

helper.push.channel_subscribe = function() {
    var channel = 'all';

    parsePlugin.subscribe(channel, function() {
        storage.set('channel', channel);
        helper.push.get_installation_id();
    }, function(e) {
        console.error('Channel subscription error');
    });
};

helper.push.get_installation_id = function(callback) {
    parsePlugin.getInstallationId(function(id) {
        storage.set('installation_id', id);
        helper.push.register_rewards();
    }, function(e) {
        console.error('installation id error', e);
    });
};

helper.push.register_rewards = function() {
    // base = 'http://api.rewards-admin.dev/api/v1/user/';
    base = 'https://api.fanmaker.com/api/v1/user/';
    $.ajax({
        url: base + storage.get('user_id') + '/devices',
        dataType: 'json',
        type: 'POST',
        data: {
            'installation_id': storage.get('installation_id'),
            'os': device.version,
            'name': device.model,
            'app_name': app.name,
            'app_version': app.version
        },
        success: function(msg) {
            helper.log(msg, 'parse register success');
        },
        failure: function(msg) {
            helper.error(msg, 'parse register failure');
        }
    });
};


// http://stackoverflow.com/questions/1527803/generating-random-numbers-in-javascript-in-a-specific-range
function getRandomInt (min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

// Convert .serializeArray results into more traditional JSON object
// More compatable with how we set AJAX defaults in the prefilter
$.fn.serializeObject = function()
{
    var o = {};
    var a = this.serializeArray();
    $.each(a, function() {
        if (o[this.name] !== undefined) {
            if (!o[this.name].push) {
                o[this.name] = [o[this.name]];
            }
            o[this.name].push(this.value || '');
        } else {
            o[this.name] = this.value || '';
        }
    });
    return o;
};


// Debounce thing
(function($){var a={},c="doTimeout",d=Array.prototype.slice;$[c]=function(){return b.apply(window,[0].concat(d.call(arguments)))};$.fn[c]=function(){var f=d.call(arguments),e=b.apply(this,[c+f[0]].concat(f));return typeof f[0]==="number"||typeof f[1]==="number"?this:e};function b(l){var m=this,h,k={},g=l?$.fn:$,n=arguments,i=4,f=n[1],j=n[2],p=n[3];if(typeof f!=="string"){i--;f=l=0;j=n[1];p=n[2]}if(l){h=m.eq(0);h.data(l,k=h.data(l)||{})}else{if(f){k=a[f]||(a[f]={})}}k.id&&clearTimeout(k.id);delete k.id;function e(){if(l){h.removeData(l)}else{if(f){delete a[f]}}}function o(){k.id=setTimeout(function(){k.fn()},j)}if(p){k.fn=function(q){if(typeof p==="string"){p=g[p]}p.apply(m,d.call(n,i))===true&&!q?o():e()};o()}else{if(k.fn){j===undefined?e():k.fn(j===false);return true}else{e()}}}})(jQuery);

// Equivilant of php's nl2br
function nl2br (str) {
    return (str + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + '<br />' + '$2');
}
