/**
 * This is a straightforward port from Rewards's mobile registration JS.
 * It's outside of Knockout because the layouts are rendered remotely and piped in
 * as HTML
 * 
 */
var Registration = {

    total_panels: 0,

    init: function() {
        var restore_registration_step = parseInt($(".registration-area").attr('data-restore-registration-step'));

        Registration.total_panels = parseInt($(".registration-area").attr('data-panel-count'));

        $('#main')
            .on('click', '.registration-next-button', function() {
                Registration.handle_next($(".registration-panel").filter(":visible")[0]);
            })
            .on('click', '.oauth-link', function(event){
                event.preventDefault();

                var $self = $(this),
                    path  = $self.attr('href'),
                    browser;

                helper.oauth_connection_handler(path, $self, function(callback_data) {
                    Registration.handle_next($(".registration-panel").filter(":visible")[0]);
                }, Registration);

                helper.refreshScroll();
            })
        ;
    
        if (restore_registration_step > 0)
        {
            Registration.show();
            Registration.show_page(restore_registration_step);

            $(".mobile-error").show();
        }
        else {
            Registration.show();
        }

        Registration.special_rules();
    },

    special_rules: function() {
        // For initalizing special behaviors of extra fields

        // Student ID "I am a student" checkbox
        $("#is_student").change(function() {
            if ($("#is_student").prop('checked'))
            {
                $("#student_id_picker").slideDown('slow');
            }
            else
            {
                $("#student_id_picker").slideUp('slow');
            }
        });
    },

    show : function() {

        $('#auth-content').hide();
        $('#registration-container').show();

        Registration.show_page(0);
    },

    show_page: function(page_index)
    {
        helper.log(page_index, "Page index from show_page param");
        // Special case - if a panel has an element with class 'panel-connected' then skip that panel.

        var page_search_index = page_index,
            found_ok_panel = false,
            panel,
            $registration_area = $(".registration-area"),
            $progression = $(".progression")
        $footer = $("footer");

        // Find the next panel which is incomplete - or advance user to profile page
        while (!found_ok_panel)
        {
            if (page_search_index >= Registration.total_panels)
            {
                // User has filled out everything, show profile
                helper.log("Attempted to navigate to /?m=1. page_search_index: "+ page_search_index +", total_panels: " + Registration.total_panels);

                // Refresh page, go to profile
                app.Display("welcome");
                return;
            }

            panel = $registration_area.find('#registration-panel-'+page_search_index);

            if (panel.find('.panel-connected').length == 0)
            {
                found_ok_panel = true;
            }
            else
            {
                page_search_index++;
            }
        }

        $registration_area.find('.registration-panel').hide();
        $registration_area.find('#registration-panel-'+page_search_index).show();
        $progression.find(".active").removeClass("active");
        $progression.find(".page-"+page_search_index).addClass("active");

        $(".mobile-error").hide();
    },

    handle_next: function(context) {
        // Handle progressing to the next screen, make sure all required fields are filled out first
        helper.log(context, "Handle Next for context");
        // Send page to server for processing on success

        var form = $(context);

        var validation = this.validate_panel(form);

        if (validation != 'OK')
        {
            form.find('.form-error').html(validation);
            form.find('.form-error').show();
            return;
        }
        else
        {
            form.find('.form-error').html('').hide();;

            values = [];

            form.find(".registration-input").each(function() {
                values[$(this).attr('id')] = $(this).val();
            });

            form.find(".registration-input-checkbox").each(function() {
                values[$(this).attr('id')] = $(this).prop('checked') ? 1 : 0;
            });

            // Convert array to object
            values = $.extend({}, values);

            // Convert object to json

            values = JSON.stringify(values);

            $.ajax({
                type: 'POST',
                url: helper.apiEndpoint('register'),
                dataType: 'json',
                data: {
                    page: form.attr('data-panel-number'),
                    values: values
                },
                success: function(data) {
                    var current_panel,
                        newPanelIndex;

                    if (data.error == true)
                    {
                        form.find('.form-error').html(data.validation).show();
                    }
                    else
                    {

                        // If the server sends us a session_id, save it.
                        if(data.session_id !== null) {
                            helper.log_in(data.session_id);
                        }
                        // Todo: progress to next panel
                        current_panel = form.attr('data-panel-number');

                        if (current_panel == Registration.total_panels - 1)
                        {
                            // Refresh page, go to profile
                            app.generalBindings();
                            app.Display("welcome");
                        }
                        else
                        {
                            newPanelIndex = parseInt(current_panel) + 1;
                            helper.log("newPanelIndex: "+ newPanelIndex +", current_panel: " + current_panel);
                            Registration.show_page(newPanelIndex, true);
                        }

                        // helper.log(data);

                        if(data.user_id !== null){ storage.set('user_id', data.user_id); }
                        if(data.signing_key !== null){ storage.set('signing_key', data.signing_key); }
                    }
                }
            });

        }
    },

    validate_panel: function(panel_jquery_element) {
        var error = 'OK';
        panel_jquery_element.find(".registration-input").each(function() {
            if(
                typeof($(this).attr('data-required-error')) != 'undefined' &&
                    $(this).val() == '' &&
                    error == 'OK'
                )
            {
                error = $(this).attr('data-required-error');
            }
        });

        return error;
    }
};