app.ViewModels.activity = function(data, state) {
  var self    = this;

  self.template    = "activity";
  self.user        = ko.observable();
  self.activities  = ko.observableArray();
  self.adjustments = ko.observableArray();
  self.activitiesRequestOutbound = ko.observable(false);
  self.adjustmentsRequestOutbound = ko.observable(false);

  self.allActions = ko.observableArray();

  self.pullRefresh = true;
  self.refreshIcon = ko.observable('fa-refresh');
  self.refreshOffset = ko.observable(0);

  self.scrollLoad = true;

  self.adjustmentsPage = ko.computed(function(){
    if(typeof app.View['adjustments']() !== 'undefined' && app.View['adjustments']().hasOwnProperty('pagination')) {
      return app.View['adjustments']().pagination.next_page_number;
    }

    else { return null; }
  });

  self.activitiesPage = ko.computed(function(){
    if(typeof app.View['activity']() !== 'undefined' && app.View['activity']().hasOwnProperty('pagination')) {
      return app.View['activity']().pagination.next_page_number;
    }

    else { return null; }
  });

  self.clearCurrent = true;

  self.ad    = ko.observable(app.State.ad);
  self.alert = ko.observable(app.State.alertbar);

  helper.checkPermissions();

  self.nextPage = function() {
    self.clearCurrent = false;

    self.dontCalculate = true;
    self.adjustments([]);
    self.activities([]);

    if(self.adjustmentsPage()) { self.adjustmentsRequestOutbound(true); }
    if(self.activitiesPage()) { self.activitiesRequestOutbound(true); }

    self.dontCalculate = false;

    if(self.adjustmentsPage()) {
      adjParams = { 'per_page': 20, 'page': self.adjustmentsPage() };
      helper.requestData('adjustments', true, adjParams, 'admin', 2, true);
    }

    if(self.activitiesPage()) {
      actParams = { 'per_page': 20, 'page': self.activitiesPage() };
      helper.requestData('activity', true, actParams, 'admin', 2, true);
    }

    if(!self.adjustmentsPage() && !self.activitiesPage()) {
      app.State.overLoad(false);
    }
  }

  self.getDate = function(obj) {
    var d = new Date(obj);
    return d.getDate();
  }

  self.getMonth = function(obj) {
    var d = new Date(obj);

    var monthNames = ["Jan.", "Feb.", "Mar.", "Apr.", "May", "June",
      "July", "Aug.", "Sept.", "Oct.", "Nov.", "Dec."];

    return monthNames[d.getMonth()];
  }

  self.computeActions = function(active, adjust) {
    act = active.concat(adjust);

    sorted = act.sort(function(a, b) {
      if(a.date() == "") { a = { date: function(){ return "1977-01-01T00:00:00Z"; } }; }
      if(b.date() == "") { b = { date: function(){ return "1977-01-01T00:00:00Z"; } }; }

      var dateA=new Date(a.date()), dateB=new Date(b.date());
      return dateB-dateA; //sort by date descending
    });

    if(self.clearCurrent) { self.allActions(sorted); }
    else {
      if(sorted.length){
        self.allActions(self.allActions().concat(sorted));
      }
      self.dontCalculate = true;
      self.adjustments([]);
      self.activities([]);
      self.dontCalculate = false;
    }

    window.myScroll.refresh();
  }

  self.actions = ko.computed(function() {
    if(!self.activitiesRequestOutbound() && !self.adjustmentsRequestOutbound() && !self.dontCalculate) {
      self.computeActions(self.activities(), self.adjustments());
    }
  });

  // Adjustments ---------------------------------------------------------------------------------------------

  params = { 'per_page': 20, 'page': 1 };
  helper.requestData('adjustments', true, params, 'admin', 2);

  app.View['adjustments'].subscribe(function(adjustments) {
    helper.log(adjustments, 'WHATCHA BUYING?');

    if(adjustments.status == 200) {
      tempAdjustments = [];

      if(adjustments.data.length > 0) {
        for(var i in adjustments.data) {
          tempAdjustments.push(new app.ViewModels.adj(adjustments.data[i], state, self));
        }
        self.adjustments(tempAdjustments);
        self.adjustmentsRequestOutbound(false);
      }
    }

    self.adjustmentsRequestOutbound(false);

    helper.refreshScroll();
    setTimeout(function(){ helper.refreshScroll(); }, 4000);
  });

  // display the cache
  adjustments = helper.get_view('adjustments');
  if(adjustments) { app.View['adjustments'](adjustments); }


  // Activities -----------------------------------------------------------------------------------------------

  params = { 'per_page': 20, 'page': 1 };
  helper.requestData(self.template, true, params, 'admin', 2);

  app.View[self.template].subscribe(function(activities) {
    helper.getUserDetails();
    helper.log(activities, 'WHATCHA BEEN UP TO?');

    if(activities.status == 200) {
      tempActivities = [];

      if(activities.data.length > 0) {
        for(var i in activities.data) {
          tempActivities.push(new app.ViewModels.act(activities.data[i], state, self));
        }
        self.activities(tempActivities);
        self.activitiesRequestOutbound(false);
      }
    }

    self.activitiesRequestOutbound(false);

    helper.refreshScroll();
    setTimeout(function(){ helper.refreshScroll(); }, 4000);
  });

  // display the cache
  activities = helper.get_view(self.template);
  if(activities) { app.View[self.template](activities); }


  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ADJUSTMENT VIEW
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.adj = function(data, state, root) {
  var self   = this;

  self.subTemplate = "tempAdjustment";
  self.date = ko.observable(data.date);
  self.inferred_source = ko.observable(data.inferred_source);
  self.points  = ko.observable(data.points);
  self.reason = ko.observable(data.reason);
  self.type = ko.observable(data.type);
  self.icon = ko.observable(data.icon);


  self.pointsSpent = function() {
    term = self.points() > 0 ? "Earned " : "Spent ";
    return term + helper.punctuate_number(Math.abs(self.points())) + self.delineate(self.points());
  }

  self.delineate = function(value) {
    return Math.abs(value) == 1 ? " Point" : " Points";
  }

  self.when = function() {
    return root.getMonth(self.date()) + " " + root.getDate(self.date());
  }

  self.css_classes = function() {
    if(typeof self.icon() !== "undefined" && self.icon() !== "") {
      return self.icon();
    }

    else {
      var icon = {
        purchase: 'fa-gift ptext'
      }

      if(icon.hasOwnProperty(self.type())){
        ico = icon[self.type()];
      }
      else {
        ico = 'fa-minus-circle';
      }

      return ico;

    }
  }

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ACTIVITY VIEW
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.act = function(data, state, root) {
  var self   = this;

  self.subTemplate = "tempActivity";
  self.type = ko.observable(data.type);
  self.subtype = ko.observable(data.subtype);
  self.subject = ko.observable(data.subject);
  self.link  = ko.observable(data.link);
  self.date = ko.observable(data.datetime);
  self.points_awarded = ko.observable(data.points_awarded);
  self.helptext = ko.observable(data.helptext);
  self.icon = ko.observable(data.icon);


  self.points = function() {
    return 'Earned ' + helper.punctuate_number(self.points_awarded()) + self.delineate(self.points_awarded());
  }

  self.delineate = function(value) {
    return value == 1 ? " Point" : " Points";
  }

  self.when = function() {
    return root.getMonth(self.date()) + " " + root.getDate(self.date());
  }

  self.helptextModal = function() {
    if(self.helptext()) {
      helper.alertModal(self.helptext(), 'fa-info');
    }
  }

  self.css_classes = function() {
    if(typeof self.icon() !== "undefined" && self.icon() !== "") {
      return self.icon();
    }

    else {
      var icon = {
        event: {
          default: 'fa-plus-circle',
          attendance: 'fa-check-circle'
        }
      }

      if(icon.hasOwnProperty(self.type())){
        if(icon[self.type()].hasOwnProperty(self.subtype())) { ico = icon[self.type()][self.subtype()] }
        else { ico = icon[self.type()].default; }
      }
      else {
        ico = 'fa-plus-circle';
      }

      if(self.points_awarded() == 0 || self.points_awarded() == "0") { ico = 'fa-circle-o';  }

      return ico;

    }
  }

  return self;
}