app.ViewModels.prizeFormat = [];
app.ViewModels.checkoutFormat = [];

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// AUCTION TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.prizeFormat['auction_template'] = function(data) {
	self.ending_at = data.ending_at;
  self.pickup_enabled = data.pickup_enabled;
  self.shipping_enabled = data.shipping_enabled;
  self.high_bid = ko.observable(data.high_bid);
	self.points = function(){
    return (self.high_bid()) ? self.high_bid() : data.starting_bid;
  }
  self.id = data.id;
  self.bid_history = ko.observableArray();
  self.delivery_value = ko.observable();
  self.has_bid = ko.observable(false);
  self.refreshing_history = ko.observable(false);
  self.retrieval = ko.observable(false);

  app.State.overLoad(true);

	self.description = data.description;
	self.terms_conditions = data.terms_conditions;

  params = {};
  stringifed = JSON.stringify($.extend(params, app.defaultAjaxParams()));

  self.update_history = function(){
    if(!self.refreshing_history()){
      self.refreshing_history(true);

      $.ajax({
        url: helper.apiEndpointV1('bid_history', self.id),
        method: 'get',
        dataType: 'json',
        beforeSend: function(request) {
          helper.headers(request);

          if(!helper.isLoading()) { app.State.overLoad(true); }
        },
        success: function(ret) {
          if(ret.result !== 'error' && ret.status == 'success') {
            self.bid_history(ret.data.auction.bid_history.slice(0, 30));

            if(typeof ret.data.max_bid !== 'undefined')
              self.has_bid(true);

            if(typeof ret.data.auction.high_bid !== 'undefined')
              self.high_bid(ret.data.auction.high_bid);

            if(typeof ret.data.shipping !== 'undefined')
              self.retrieval(ret.data.shipping);

            helper.log(ret, 'AUCTION STATS SUCCESS');
          }
        },
        error: function(ret) {
          helper.log(ret, 'AUCTION STATS FAILURE');
        },
        complete: function() {
          app.smallLoad(false);
          app.State.overLoad(false);
          self.refreshing_history(false);
          helper.refreshScroll();
        }
      });
    }
  }

  self.update_history();

  self.format_name = function(name) {
    if(name == "My Bid")
      return name;
    else
      return "Bidder #" + name;
  }

  self.delineate = function(value) {
    return value == 1 ? " Point" : " Points";
  }

	self.cost = ko.computed(function(){
		if(self.points() == 0) { return "No Starting Bid"; }

		var delineator = self.delineate(self.points());
		var type = self.high_bid() ? "Current Bid: " : "Starting Bid: ";
		return type + helper.punctuate_number(self.points()) + delineator;
	});

  self.pointsRemaining = ko.pureComputed(function(){

    if(root.delivery_choice()) {
      del = self.delivery_value();
    }
    else {
      del = root.delivery_method();
    }

    t = app.State.user().point_total() + "";
    total = parseFloat(t.replace(',','').replace(/[^\d\.\-]/g, ""));

    quantity = self.amount() ? self.amount() : 1;
    answer = total - (self.points() * quantity);
    if(del == "Delivery" || del == "delivery") { answer = answer - self.shipping_cost_points }
    answer = helper.punctuate_number(answer) + self.delineate(answer);

    return answer;

  });

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// AUCTION CHECKOUT TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.checkoutFormat['auction_checkout'] = function(data) {
  self.bid_increment = data.bid_increment;
  self.starting_bid = data.starting_bid;

  self.min_bid = function(){
    if(self.high_bid())
      return self.high_bid() + self.bid_increment + self.delineate(self.high_bid() + self.bid_increment);
    else
      return self.starting_bid + self.delineate(self.starting_bid);
  }

  self.submit_bid = function(){
    max_bid = $('#max_bid').val();
    params = { 'points': max_bid, 'user_id': helper.user_id() };

    if(max_bid == "") { helper.alertModal('Please enter a max bid!', 'fa-exclamation-triangle'); return; }

    helper.confirmModal("Are you sure you want to place a bid for "+max_bid+self.delineate(max_bid)+"?", "fa-gavel", "Yes", "No", function(){
      submit = true;
      if(!self.has_bid()) {
        if(self.delivery_value() !== "pickup"){
          $.each($("#shippingInfo input:not(input[name='addr2'])"), function(){
            if($(this).val() == '') { helper.alertModal('Please make sure your contact info is correct so that we can reach you if you win!', 'fa-exclamation-triangle'); submit = false; }
          });
        }
        if(submit) self.address_request();
      }
      if(submit) self.bid_request(params);
    });
  }

  self.address_request = function() {
    shipping = {
      auction_id: self.id,
      name: $('input[name="name"]').val(),
      address1: $('input[name="addr1"]').val(),
      address2: $('input[name="addr2"]').val(),
      city: $('input[name="city"]').val(),
      state: $('input[name="state"]').val(),
      zip: $('input[name="zip"]').val(),
      phone: $('input[name="phone"]').val(),
      pickup: self.delivery_value() == "pickup" ? true : false
    };

    $.ajax({
      url: helper.apiEndpointV1('shipping', self.id),
      method: 'post',
      data: { 'shipping': shipping },
      dataType: 'json',
      beforeSend: function(request) {
        helper.headers(request);
        if(!helper.isLoading()) { app.State.overLoad(true); }
      },
      success: function(ret) {
        if(ret.result !== 'error' && ret.status == 'success') {
          helper.log(ret, 'SHIPPING SUCCESS');
        }
      },
      error: function(result) {
        ret = JSON.parse(result.responseText);
        helper.log(ret, 'SHIPPING FAILURE');
      }
    });
  }


  self.bid_request = function(params) {
     $.ajax({
        url: helper.apiEndpointV1('submit_bid', self.id),
        method: 'post',
        data: params,
        dataType: 'json',
        beforeSend: function(request) {
          helper.headers(request);
          if(!helper.isLoading()) { app.State.overLoad(true); }
        },
        success: function(ret) {
          if(ret.result !== 'error' && ret.status == 'success') {
            postSubmission();

            if(ret.data.max_bid.is_leader) { helper.alertModal('You are the current high bidder!'); }

            self.high_bid(ret.data.auction.high_bid);
            helper.log(ret, 'AUCTION BID SUCCESS');
          }
        },
        error: function(result) {
          ret = JSON.parse(result.responseText);
          postSubmission();

          self.high_bid(ret.data.auction.high_bid);
          if(ret.messages) helper.alertModal(ret.messages[0], 'fa-exclamation-triangle');

          helper.log(ret, 'AUCTION BID FAILURE');
        }
      });

     postSubmission = function(){
      app.smallLoad(false);
      app.State.overLoad(false);
      self.update_history();
      helper.toggleCheckout();
      $('#max_bid').val('');
    }
  }

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// CONTEST TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.prizeFormat['contest_template'] = function(data) {

	self.ending_at = data.ending_at;
	self.entry_points = data.entry_points;
	self.quantity = data.quantity;
	self.description = data.description;
	self.terms_conditions = data.terms_conditions;
  self.free_first_entry_enabled = data.free_first_entry_enabled;
  self.free_first_entry_mobile_app = data.free_first_entry_mobile_app;
  self.user_entered = data.user_entered;
  self.qty = ko.observable(100);

  self.free_first_all = (self.free_first_entry_enabled && !self.free_first_entry_mobile_app && !self.user_entered && data.entry_points > 0);
  self.free_first_mobile = (self.free_first_entry_enabled && self.free_first_entry_mobile_app && !self.user_entered && data.entry_points > 0);

  self.free_first = self.free_first_all || self.free_first_mobile;

  self.grammarModifier = data.entry_points > 0 ? " Each" : "";

	self.cost = function() {
		if(data.entry_points == 0) { return "Free Entry"; }

		var delineator = self.entry_points == 1 ? " Point" : " Points";
		return helper.punctuate_number(self.entry_points) + delineator + " Per Contest Entry";
	}

	self.winners = function() {
		var delineator = data.quantity == 1 ? " Winner" : " Winners";
		return helper.punctuate_number(data.quantity) + delineator;
	}

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// CONTEST CHECKOUT TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.checkoutFormat['contest_checkout'] = function(data) {
	self.ending_at = data.ending_at;
	self.entry_points = data.entry_points;
	self.quantity = data.quantity;
  self.free_first_entry_enabled = data.free_first_entry_enabled;
  self.free_first_entry_mobile_app = data.free_first_entry_mobile_app;
  self.user_entered = data.user_entered;
	self.description = data.description;
	self.terms_conditions = data.terms_conditions;
	self.pickup_enabled = data.pickup_enabled;
	self.shipping_enabled = data.shipping_enabled;
  self.id = data.id;

  self.free_first_all = (self.free_first_entry_enabled && !self.free_first_entry_mobile_app && !self.user_entered && data.entry_points > 0);
  self.free_first_mobile = (self.free_first_entry_enabled && self.free_first_entry_mobile_app && !self.user_entered && data.entry_points > 0);

  self.free_first = self.free_first_all || self.free_first_mobile;

  self.grammarModifier = data.entry_points > 0 ? " Each" : "";

	self.amount = ko.observable(1);

	self.delineate = function(value) {
		return value == 1 ? " Point" : " Points";
	}

	self.cost = function() {
		if(data.entry_points == 0) { return "Free Entry"; }

		return helper.punctuate_number(data.entry_points) + delineate(data.entry_points);
	}

	self.pointsRemaining = ko.pureComputed(function(){

		t = app.State.user().point_total() + "";
    total = parseFloat(t.replace(',','').replace(/[^\d\.\-]/g, ""));

		quantity = self.amount() ? self.amount() : 1;
    if(self.free_first || self.free_first_mobile) { quantity = quantity - 1; }
		answer = total - (self.entry_points * quantity);
		answer = helper.punctuate_number(answer) + self.delineate(answer);

		return answer;

	});

  self.redemption = function() {
    try {
      quantity = parseInt($('#contestQuantity').val());
      if(isNaN(quantity)) throw "No Quantity";
    }
    catch(e) {
      quantity = 1;
    }

    if(data.pickup_enabled && data.shipping_enabled && data.shipping_cost_type !== "card") {
      var delivery = $('.deliveryMethod').val();
    }

    else {
      var delivery = self.pickup_enabled ? "pickup" : "delivery";
    }

    var name = $('input[name="name"]').val(),
        addr1 = $('input[name="addr1"]').val(),
        addr2 = $('input[name="addr2"]').val(),
        city = $('input[name="city"]').val(),
        state = $('input[name="state"]').val(),
        zip = $('input[name="zip"]').val(),
        phone = $('input[name="phone"]').val();

    params = { "quantity" : quantity, "delivery_method" : delivery, "name" : name, "address1" : addr1, "address2" : addr2, "city" : city, "state" : state, "zip" : zip, "phone" : phone, "mobile_app" : true };

    stringifed = JSON.stringify($.extend(params, app.defaultAjaxParams()));

    $.ajax({
      url: helper.apiEndpointV2('redeem_contest', self.id),
      data: params,
      method: 'post',
      dataType: 'json',
      contentType: "application/json",
      beforeSend: function(request) {
        helper.headers(request, stringifed);

        if(!helper.isLoading()) { app.State.overLoad(true); }
      },
      success: function(ret) {
        if(ret.result !== 'error') {
          helper.log(ret, 'CONTEST REDEMPTION SUCCESS');

          switch(ret.status) {
            case 200:
              if(ret.data.entries_purchased > 0) {
                if(ret.data.message) { helper.alertModal(ret.data.message, 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }
                else if(ret.message) { helper.alertModal(ret.message, 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }
                else { helper.alertModal("You have successfully entered the contest!", 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }
              }

              else {
                helper.log([ret, ret.message], 'WHAT IS THIS?');
                if(ret.data.message) { helper.alertModal(ret.data.message); }
                else if(ret.message) { helper.alertModal(ret.message); }
                else { helper.alertModal("We were unable to enter you into the contest, please try again.", 'fa-exclamation-triangle'); }
              }

              break;

            default:
              if(ret.message) { helper.alertModal(ret.message, 'fa-trophy', 'Dismiss','', helper.toggleCheckout); }
              else { helper.alertModal("We were unable to enter you into the contest, please try again.", 'fa-exclamation-triangle', 'Dismiss','', helper.toggleCheckout); }

              break;
          }
        }
      },
      error: function(ret) {
        helper.log(ret, 'CONTEST REDEMPTION ERROR');

        helper.alertModal("We were unable to enter you into the contest, please try again.", 'fa-exclamation-triangle');
      },
      complete: function() {
        app.smallLoad(false);
        app.State.overLoad(false);
      }
    });
  }

  return self;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ITEM TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.prizeFormat['item_template'] = function(data) {

  // -------------------------------------------------- Variables --------------------------------------------------

	self.points = data.cost_points;

	self.description = data.description;
	self.terms_conditions = data.terms_conditions;
  self.id = data.id;
  self.is_unlock = data.is_unlock;
  var delineator = self.points == 1 ? " Point" : " Points";

  self.unlock_message = helper.crossedThreshold(self.cost_points) ? 'Unlocked' : 'Earn at least ' + helper.punctuate_number(self.points) + delineator + " to Unlock";

  // -------------------------------------------------- Functions --------------------------------------------------

  self.item_url = function(url) {
    window.open(url, '_system');
  }

	self.cost = function() {
		if(!self.points || self.points == 0) { return "No Points Required"; }

		return helper.punctuate_number(self.points) + delineator;
	}

  self.redemption = function() {
    // try {
    //   quantity = parseInt($('#itemQuantity').val());
    //   if(isNaN(quantity)) throw "No Quantity";
    // }
    // catch(e) {
    //   quantity = 1;
    // }

    if(data.pickup_enabled && data.shipping_enabled && data.shipping_cost_type !== "card") {
      var delivery = $('.deliveryMethod').val();
    }

    else {
      var delivery = self.pickup_enabled ? "pickup" : "delivery";
    }

    var name = $('input[name="name"]').val(),
        addr1 = $('input[name="addr1"]').val(),
        addr2 = $('input[name="addr2"]').val(),
        city = $('input[name="city"]').val(),
        state = $('input[name="state"]').val(),
        zip = $('input[name="zip"]').val(),
        phone = $('input[name="phone"]').val();

    params = { "delivery_method" : delivery, "name" : name, "address1" : addr1, "address2" : addr2, "city" : city, "state" : state, "zip" : zip, "phone" : phone, "mobile_app" : true };

    if(self.item_options.length > 0) {
      params = $.extend(params, { 'option_id': $('.itemOption').val() });
    }

    stringifed = JSON.stringify($.extend(params, app.defaultAjaxParams()));

    $.ajax({
      url: helper.apiEndpointV2('redeem_item', self.id),
      data: params,
      method: 'post',
      dataType: 'json',
      contentType: "application/json",
      beforeSend: function(request) {
        helper.headers(request, stringifed);

        if(!helper.isLoading()) { app.State.overLoad(true); }
      },
      success: function(ret) {
        if(ret.result !== 'error') {
          helper.log(ret, 'ITEM REDEMPTION SUCCESS');

          switch(ret.status) {
            case 200:
              if(ret.data.delivery_message) { helper.alertModal(ret.data.delivery_message, 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }
              else if(ret.message) { helper.alertModal(ret.message, 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }
              else { helper.alertModal("You have successfully redeemed this item!", 'fa-thumbs-o-up', 'Dismiss', '', helper.toggleCheckout); }

              break;

            default:
              if(ret.message) { helper.alertModal(ret.message, 'fa-trophy', 'Dismiss','', helper.toggleCheckout); }
              else { helper.alertModal("We were unable to redeem this item for you, please try again.", 'fa-exclamation-triangle', 'Dismiss','', helper.toggleCheckout); }

              break;
          }
        }
      },
      error: function(ret) {
        helper.log(ret, 'CONTEST REDEMPTION ERROR');

        helper.alertModal("We were unable to redeem this item for you, please try again.", 'fa-exclamation-triangle');
      },
      complete: function() {
        app.smallLoad(false);
        app.State.overLoad(false);
      }
    });
  }

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ITEM CHECKOUT TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.checkoutFormat['item_checkout'] = function(data, root) {
  self.points = data.cost_points;

  self.description = data.description;
  self.terms_conditions = data.terms_conditions;
  self.id = data.id;

  self.quantity = data.quantity;
  self.redemption_limit = data.redemption_limit;

  if(self.redemption_limit){
    limit = self.redemption_limit < self.quantity ?
      self.redemption_limit : self.quantity;
  }
  else {
    limit = self.quantity;
  }

  self.limit = ko.observable(limit);
  self.pickup_enabled = data.pickup_enabled;
  self.shipping_enabled = data.shipping_enabled;
  self.shipping_cost_type = data.shipping_cost_type;
  self.shipping_cost_points = data.shipping_cost_points;
  self.item_options = data.item_options;
  self.delivery_value = ko.observable();

  self.amount = ko.observable(1);

  self.delineate = function(value) {
    return value == 1 ? " Point" : " Points";
  }

  self.pointsRemaining = ko.pureComputed(function(){

    if(root.delivery_choice()) {
      del = self.delivery_value();
    }
    else {
      del = root.delivery_method();
    }

    t = app.State.user().point_total() + "";
    total = parseFloat(t.replace(',','').replace(/[^\d\.\-]/g, ""));

    quantity = self.amount() ? self.amount() : 1;
    answer = total - (self.points * quantity);
    if(del == "Delivery" || del == "delivery") { answer = answer - self.shipping_cost_points }
    answer = helper.punctuate_number(answer) + self.delineate(answer);

    return answer;

  });

  return self;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// PARENT PRIZE TEMPLATE
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
app.ViewModels.prize = function(data) {

	helper.log(data, 'AND I SEE DATA');

	// -------------------------------------------------- Variables --------------------------------------------------

	var self = this;
	self.template = "prize";
	self.subTemplate = data.subTemplate;
	self.checkoutTemplate = data.checkoutTemplate;

	self.shippingCost = (data.shipping_cost_type && data.shipping_cost_type == "card") ? true : false;
  self.pickup_enabled = data.pickup_enabled;
  self.shipping_enabled = data.shipping_enabled;
  self.shipping_cost_currency = data.shipping_cost_currency;
  self.shipping_cost_points = data.shipping_cost_points;
  self.shipping_cost_type = data.shipping_cost_type;
  self.quantity = data.quantity;
  self.is_unlock = data.is_unlock;
  self.cost_points = data.cost_points;

  self.redeem_button = function(){
    switch(self.subTemplate){
      case 'auction_template':
        return 'Place Bid';
        break;
      case 'contest_template':
        return 'Enter Contest';
        break;
      case 'item_template':
        if(self.is_unlock && !helper.crossedThreshold(self.cost_points))
          return 'Unlock';
        else if(self.is_unlock)
          return 'Redeem';
        else
          return 'Redeem Points';
        break;
    }
  }

	self.model = ko.observable(new app.ViewModels.prizeFormat[data.subTemplate](data));

	self.checkoutVisible = ko.observable(false);

	self.images = data.images;
	self.title = data.title;

	self.alert = ko.observable(app.State.alertbar);
	app.loading(false);


	// -------------------------------------------------- Functions --------------------------------------------------


  self.show_redeem = function() {
    if(!self.shippingCost || self.pickup_enabled && self.quantity > 0) { return true; }
    else { return false; }
  }

  self.delivery_choice = function() {
    if(self.shipping_enabled && self.pickup_enabled) {
      if(!self.shippingCost) { return true; }
      else { return false; }
    }
    else { return false; }
  }

  self.delivery_cost = function() {
    return (self.shipping_enabled && self.shipping_cost_type == 'points'
           && self.shipping_cost_points > 0) ? true : false;
  }

  self.shipping_cost_calc = function() {
    var delineator = self.shipping_cost_points == 1 ? " Point" : " Points";
    return helper.punctuate_number(self.shipping_cost_points) + delineator;
  }

  self.delivery_method = function() {
    if(!self.shippingCost && self.shipping_enabled && !self.pickup_enabled) { return 'Delivery'; }
    else if(self.pickup_enabled) { return 'Pick-Up'; }
  }

	$('img').off('load').on('load', function(){
		$.doTimeout('myscrollrefresh', 800, function() {
			helper.refreshScroll();
		});
	});

	ko.bindingHandlers.slide = {
    update: function (element, valueAccessor) {
        var value = self.checkoutVisible();
        value ? $(element).slideDown('fast', self.refresh()) : $(element).slideUp('fast', self.refresh());
    }
	};

	self.refresh = function(){
		setTimeout(function(){
			 helper.refreshScroll();
		},200);
	}

	self.back = function() {
		app.Display('prizes');
		return;
	}

	self.thisIsTheEnd = function(date){
		if(date) {
			d = new Date(date);

			return "Ends " + self.getMonth(d) + " " + self.getDay(d) + " at " + self.getTime(d);
		}
		else { return false; }
	}

	self.getMonth = function(d) {
    var monthNames = ["January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"];

    return monthNames[d.getMonth()];
  }


  self.getDay = function(d) {
  	day = d.getDate();

  	switch(day) {
  		case 1:
  		case 21:
  		case 31:
  			day = day + "st";
  			break;
  		case 2:
  		case 22:
  			day = day + "nd";
  			break;
  		case 3:
  		case 23:
  			day = day + "rd";
  			break;
  		default:
  			day = day + "th";
  			break;
  	}

    return day;
  }

  self.getTime = function(d) {

    var hours = d.getHours() % 12 == 0 ? '12' : d.getHours() % 12;
    var minutes = d.getMinutes() < 10 ? '0'+d.getMinutes() : d.getMinutes();
    var timeOfDay = d.getHours() < 12 ? "AM" : "PM";

    return hours + ":" + minutes + " " + timeOfDay;

  }

  self.redeem = function() {
    address = helper.address();
    if(address){

      $('input[name="name"]').val(address.name);
      $('input[name="addr1"]').val(address.address1);
      $('input[name="addr2"]').val(address.address2);
      $('input[name="city"]').val(address.city);
      $('input[name="state"]').val(address.state);
      $('input[name="zip"]').val(address.zip);
      $('input[name="phone"]').val(address.phone);
    }

  	helper.toggleCheckout();
	}

  self.checkout = ko.observable(new app.ViewModels.checkoutFormat[data.checkoutTemplate](data, self));

	return self;

}




