app.ViewModels.question = [];

app.ViewModels.question['QuestionFreeText'] = function(data, parent) {
  var self = this;
  self.template = ko.observable('QuestionFreeText');
  self.display_type = ko.observable(data.question.display_type);
  self.character_limit = ko.observable(data.question.response_charachter_limit);
  self.id = ko.observable(data.question.id);

  parent.points(data.question.points);
  parent.text(data.question.text);

  return self;
}

app.ViewModels.question['QuestionRating'] = function(data, parent) {
  var self = this;
  self.template = ko.observable('QuestionRating');
  self.display_type = ko.observable(data.question.display_type);
  self.id = ko.observable(data.question.id);

  parent.points(data.question.points);
  parent.text(data.question.text);

  self.star_text = ko.observable('');
  self.star_text_options = ko.observableArray();

  for (var i = 0; i < data.response_options.length; i++) {
    self.star_text_options.push(data.response_options[i].value);
  };

  // Logic for the stars
  self.clicked_star = function(star){
    $('.question-star').addClass('fa-star-o').removeClass('fa-star');

    for (var i = star; i >= 0; i--) {
      $('.question-star[data-id="'+i+'"]').removeClass('fa-star-o').addClass('fa-star');
    };

    self.star_text(self.star_text_options()[star]);
  };

  return self;
}

app.ViewModels.question['QuestionMultipleChoice'] = function(data, parent) {
  var self = this;
  self.template = ko.observable('QuestionMultipleChoice');
  self.display_type = ko.observable(data.question.display_type);
  self.id = ko.observable(data.question.id);
  self.response_options = ko.observable(data.response_options);

  parent.points(data.question.points);
  parent.text(data.question.text);

  return self;
}

app.ViewModels.survey = function() {
  var self = this;

  self.submit_button_text = ko.observable('Submit');

  self.points = ko.observable(0);
  self.text = ko.observable('');
  self.modal = ko.observable(app.State.modal);

  self.updateContent = function(result) {
    helper.log(result);

    self.banner = ko.observable(app.State.questionbar);

    self.result = ko.observable(result);
    self.type = ko.observable(result.data.type);

    self.survey_id = ko.observable(result.data.question.micro_survey_id);
    self.question = ko.observable(new app.ViewModels.question[result.data.type](result.data, self));

    if(result.data.pop_modal) {
      self.modal().active(result.data.pop_modal);
      self.updateQuestion(self.question().id());
    }
    else {
      self.banner().displayBanner(helper.site_blob().settings);
    }
  }

  // Skipping the question.
  self.skip = function() {
    self.modal().active(false);
    self.postSkip();
  };

  self.updateQuestion = function(id) {
    $.ajax({
      beforeSend: function (request){ helper.headers(request); },
      url: helper.apiEndpointV1("question_update", id),
      method: "PUT",
      data: { "user_id" : helper.user_id(), "site_id" : helper.site_id(),
      "question_id" : self.question().id(), "requested_at" : new Date().getTime() },
      success: function(result) {
        if(result.success){
          if(result.messages.lentgh > 0) {
            helper.log(result.messages[0]);
          }
        }
        else {
          if(result.messages.lentgh > 0) {
            helper.log(result.messages[0]);
          }
        }
      }
    });
  }

  // Get the response
  self.submit = function() {
    var answer;
    switch(self.type()){
      case 'QuestionRating':
        answer = self.question().star_text_options()[$('.question-rating .question-star.fa-star').last().data('id')];
        if(answer == null) { helper.warnInModal('Please select a rating.'); return; }
        break;
      case 'QuestionFreeText':
        if(self.question().display_type() == 'text-paragraph'){
          answer = $('.question-response-textarea textarea').val();
        }
        else {
          answer = $('.question-response-text input').val();
        }
        break;
      case 'QuestionMultipleChoice':
        answer = $('.question-response-dropdown select').val();
        break;
    }

    if(answer == null || answer == "" || typeof answer == 'undefined') {
      // Sometimes the app screws up the template if both
      // types of modals are opened around the same time
      if(app.State.modal.template() == "genericModal") {
        app.State.modal.content(app.State.genericModal);
        app.State.genericModal.submit();
      }
      helper.warnInModal('Please answer the question to earn points!');
    }
    else {
      self.respond(self.question().id(), answer);
      self.modal().active(false);
    }
  };

  // Post the response
  self.respond = function(id, answer) {
    $.ajax({
      beforeSend: function (request){ helper.headers(request); },
      url: helper.apiEndpointV1("question_response", id),
      method: "POST",
      data: { "response_value" : answer,  "user_id" : helper.user_id(),
      "site_id" : helper.site_id(), "question_id" : self.question().id(),
      "requested_at" : new Date().toISOString() },
      success: function(result) {

        helper.log(result);

        if(result.success){
          if(result.messages.length > 0) {
            for (var i in result.messages) {
              helper.alertModal(result.messages[i], 'fa-thumbs-up')
            };
          }
        }
        else {
          if(result.messages.length > 0) {
            for (var i in result.messages) {
              helper.alertModal(result.messages[i]);
            };
          }
        }
      },
      complete: function(){
        setTimeout(function(){ helper.checkQuestion(); }, 2000);
      }
    });
  }

  // Post the question with a null value
  self.postSkip = function() {
    $.ajax({
      beforeSend: function (request){ helper.headers(request); },
      url: helper.apiEndpointV1("question_response", self.question().id()),
      method: "POST",
      data: { "user_id" : helper.user_id(),
      "site_id" : helper.site_id(), "question_id" : self.question().id(),
      "requested_at" : new Date().toISOString() },
      success: function(result) {
        if(result.success){
          if(result.messages.length > 0) {
            for (var i in result.messages) {
              helper.alertModal(result.messages[i], 'fa-thumbs-up');
            };
          }
        }
        else {
          if(result.messages.length > 0) {
            for (var i in result.messages) {
              helper.alertModal(result.messages[i]);
            };
          }
        }
      },
      complete: function(){
        setTimeout(function(){ helper.checkQuestion(); }, 2000);
      }
    });
  }

	return self;
}


