app.ViewModels.upcoming = function(data, state) {
  var self = this;
  self.template = "upcoming";
  self.events = ko.observableArray([]);
  self.quantity = ko.observable(0);
  self.refreshOffset = ko.observable(0);

  helper.checkPermissions();

  // self.user = ko.observable(new app.ViewModels.user({}, state));
  self.ad   = ko.observable(app.State.ad);
  self.alert = ko.observable(app.State.alertbar);

  $('body').on('click', '.eventDescription a', function(e){
    e.preventDefault();
    link = $(e.target).attr('href');
    window.open(link, "_system");
  });

  $('body').on('click', '.permissionsHelp', function(e){
    helper.Jeeves['location']();
  });

  self.navigate = function(model, event) {
    var target = $(event.currentTarget).attr("href");
    app.Display(target);
  };

  blob = helper.site_blob();
  qty = blob.upcomingEventsCount ? blob.upcomingEventsCount : 20;
  date = new Date().addHours(-6).toISOString();
  params = { 'per_page': qty, 'order': 'asc', 'after': date };
  helper.requestData(self.template, true, params, 'admin', 2);

  app.loading(true);

  self.subscription = app.View[self.template].subscribe(function(upcoming) {
    helper.log(upcoming, 'UPCOMING DATA');
    helper.getUserDetails();

    self.events([]);
    if(typeof upcoming.data == "object"){ self.quantity(upcoming.data.length); }
    else { self.quantity(0); }

    for(var i in upcoming.data) {
      self.events.push(new app.ViewModels.upcoming_event(upcoming.data[i], state));
    }

    app.loading(false);
    helper.refreshScroll();
    setTimeout(function(){ helper.refreshScroll(); }, 4000);

    if(self.events().length > 0){
      self.events()[0].showDetails(true);
      helper.auto_checkin(self.events());
    }
  });

  // display the cache
  upcoming = helper.get_view(self.template);
  if(upcoming) {
    app.loading(false);
    app.View[self.template](upcoming);
  }

  app.layout('full');
  return self;
}

app.ViewModels.upcoming_event = function(data, state) {
  var self = this;

  self.user_checkin_points     = ko.observable(data.user_checkin_points);
  self.bonus                   = data.bonus;
  self.banner_image            = data.banner_image;
  self.checked_in              = ko.observable(data.checked_in);
  self.checkins_end_at         = data.checkins_end_at;
  self.checkins_start_at       = data.checkins_start_at;
  self.description             = data.description;
  self.early_arrival_bonus     = data.early_arrival_bonus;
  self.early_arrival_ends_at   = data.early_arrival_ends_at;
  self.stay_late_bonus         = data.stay_late_bonus;
  self.stay_late_starts_at     = data.stay_late_starts_at;
  self.event_name              = data.event_name;
  self.id                      = data.id;
  self.location                = data.custom_location_name || data.venues[0];
  self.method_points           = data.method_points;
  self.min_points              = data.points + data.bonus;
  self.points                  = data.points;
  self.starts_at               = data.starts_at;
  self.ticket_purchase_url     = data.ticket_purchase_url;
  self.checkingin              = ko.observable(false);
  self.showDetails             = ko.observable(false);
  self.unfindable              = ko.observable(false);
  self.distance                = ko.observable(false);

  self.template = "event";

  self.status = ko.observable(false);
  self.status_class = ko.observable('hidden');
  self.checkin_button_class = ko.observable('');

  self.icon = "<i class='fa fa-check-circle ptext'></i>";
  self.checkedInMessage = "Attended";

  self.details = function(item, event) {
    self.showDetails(!self.showDetails());
  }

  self.banner = function(){
    if(self.banner_image) {
      return self.banner_image.indexOf('missing.png') >= 0 ? false : true;
    }

    return false;
  }

  self.checkin_message = ko.computed(function(){
    if(self.checked_in()) {
      if(self.min_points && "early_arrival" in self.user_checkin_points() && "stay_late" in self.user_checkin_points() && self.early_arrival_bonus > 0 && self.stay_late_bonus > 0)
        return "You attended and earned " +self.min_points+ " points in addition to "+self.user_checkin_points()['early_arrival']+" points for coming early, and "+self.user_checkin_points()['stay_late']+" points for staying late";
      if(self.min_points && "stay_late" in self.user_checkin_points() && self.stay_late_bonus > 0)
        return "You attended and earned " +self.min_points+ " points in addition to "+self.user_checkin_points()['stay_late']+" points for staying late";
      if(self.min_points && "early_arrival" in self.user_checkin_points() && self.early_arrival_bonus > 0)
        return "You attended and earned " +self.min_points+ " points in addition to "+self.user_checkin_points()['early_arrival']+" points for coming early";
      if(self.min_points)
        return "You attended and earned " +self.min_points+ " points";
      return "Thanks for attending";
    }
    else {
      now = new Date();
      if(self.unfindable())
        return "Your location can not be accessed, please check your permission settings or data connection";
      if(self.distance())
        return "You are not close enough to this event to check in";
      if(helper.compareTime(self.checkins_end_at, now))
        return "Please check in after "+helper.displayTime(self.checkins_start_at, 'g:i A')+" to earn points";
      if(helper.compareTime(now, self.checkins_end_at))
        return "It is too late to check into this event for points";
    }

    return false;
  });

  ko.bindingHandlers.slideShow = {
    update: function(element, valueAccessor, allBindings, viewModel, bindingContext) {
      if(self.showDetails()){
       $(element).slideDown('fast', function(){
          if(helper.compareTime(self.checkins_end_at, now)
            && helper.compareTime(now, self.checkins_start_at))
            self.manualCheckin();
          helper.refreshScroll();
        });
      }
      else {
        $(element).slideUp('fast', function(){
          helper.refreshScroll();
        });
      }
    }
  };

  self.checkinEnabled = ko.computed(function(){
    now = new Date();
    return (!self.checked_in() && helper.compareTime(self.checkins_end_at, now)
            && helper.compareTime(now, self.checkins_start_at));
  });

  self.colorize = function() {
    $('.checkinMessage').addClass('warnTextColor');
    setTimeout(function(){
      $('.checkinMessage').removeClass('warnTextColor');
    }, 2000);

  }

  self.manualCheckin = function(){
    self.checkingin(true);
    helper.manual_checkin(self.id, self);
  }

  self.getDate = function(obj) {
    var d = new Date(obj);
    return d.getDate();
  }

  self.getMonth = function(obj) {
    var d = new Date(obj);

    var monthNames = ["January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"];

    return monthNames[d.getMonth()];
  }

  self.update_variables = function(result) {
    helper.log(result, "UPDATE VARIABLES")
    self.user_checkin_points(result.user_checkin_points);
  }

  // REVISIT WITH NEW API
  self.startTime = function(obj) {
    var d = new Date(obj);
    var now = new Date();

    var hours = d.getHours() % 12 == 0 ? '12' : d.getHours() % 12;
    var minutes = d.getMinutes() < 10 ? '0'+d.getMinutes() : d.getMinutes();
    var timeOfDay = d.getHours() < 12 ? "AM" : "PM";

    bonus = "";
    if(self.early_arrival_bonus && self.early_arrival_ends_at
      && helper.compareTime(d, now) && self.early_arrival_bonus > 0) {
      helper.log(['early: ', self.early_arrival_ends_at, ' time: ', self.starts_at]);
      bonus = ", Arrive " + helper.timeDifference(self.starts_at, self.early_arrival_ends_at) + "min early for " + self.early_arrival_bonus + " points";
    }
    else if(self.stay_late_bonus && self.stay_late_starts_at
      && helper.compareTime(new Date(self.checkins_end_at), now)
      && self.stay_late_bonus > 0) {
      helper.log(['late: ', self.stay_late_starts_at, ' time: ', self.starts_at]);
      bonus = self.formatLateMessage();
    }

    return hours + ":" + minutes + " " + timeOfDay + bonus;
  }

  self.formatLateMessage = function() {
    blob = helper.site_blob();
    if(blob.stayLateText) {
      bonus = blob.stayLateText;
      bonus = bonus.replace("{time}", helper.timeDifference(self.stay_late_starts_at, self.starts_at));
      bonus = bonus.replace("{bonus}", self.stay_late_bonus);

      return bonus;
    }
    else {
      return ", Stay at least  " + helper.timeDifference(self.stay_late_starts_at, self.starts_at) + "min for " + self.stay_late_bonus + " points";
    }
  }

  self.ticketsPurchasable = function() {
    now = new Date();
    return (self.ticket_purchase_url && helper.compareTime(self.starts_at, now));
  }

  self.slideit = function(elem) {
    e = $(elem);
    e.slideToggle('fast', function(){
      helper.refreshScroll();
    });
  }

  self.calcImgHeight = function() {
    return $('body').width()/2 + 'px';
  }

  self.purchaseTickets = function() {
    window.open(self.ticket_purchase_url, "_system");
  }

  return self;
}


